Package: metap
Type: Package
Title: Meta-Analysis of Significance Values
Version: 1.14
Date: 2026-04-29
Authors@R: person("Michael", "Dewey", role = c("aut", "cre"),
    email = "lists@dewey.myzen.co.uk",
    comment = c(ORCID = "0000-0002-7522-3677")
    )
Description: The canonical way to perform meta-analysis involves using effect sizes.
   When they are not available this package provides a number of methods for
   meta-analysis of significance values including the methods of Edgington, Fisher,
   Lancaster, Stouffer, Tippett, and Wilkinson; a number of data-sets to replicate published results;
   and routines for graphical display.
Depends: R (>= 3.5.0)
Imports: lattice,Rdpack (>= 0.7),TFisher,mathjaxr (>=
        2.0-0),qqconf,mutoss
RdMacros: Rdpack,mathjaxr
BuildManual: TRUE
License: GPL-2
LazyLoad: yes
NeedsCompilation: no
Packaged: 2026-04-29 08:42:26 UTC; MichaelUser
Author: Michael Dewey [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7522-3677>)
Maintainer: Michael Dewey <lists@dewey.myzen.co.uk>
Repository: CRAN
Date/Publication: 2026-05-01 13:00:02 UTC
Built: R 4.5.2; ; 2026-05-01 18:15:00 UTC; unix

export(albatros)
export(allmetap)
export(invchisq)
export(invt)
export(logitp)
export(meanp)
export(meanz)
export(maximump)
export(minimump)
export(plotp)
export(schweder)
export(sumlog)
export(sump)
export(sumz)
export(truncated)
export(two2one)
export(votep)
export(wilkinsonp)
S3method(print,allmetap)
S3method(print,invchisq)
S3method(print,invt)
S3method(print,logitp)
S3method(print,meanp)
S3method(print,meanz)
S3method(print,sumlog)
S3method(print,sump)
S3method(print,sumz)
S3method(print,votep)
S3method(print,maximump)
S3method(print,minimump)
S3method(print,truncated)
S3method(summary,truncated)
S3method(plot,truncated)
S3method(print,wilkinsonp)
S3method(plot,metap)
importFrom("graphics", "abline", "axis", "lines", "mtext", "plot", "text")
importFrom("stats", "binom.test", "coef", "lm", "na.fail", "pbeta", "pchisq",
   "pnorm", "ppoints", "pt", "qbeta", "qchisq", "qnorm", "qqline", "qqplot",
   "qunif", "qt", "sd", "spline")
importFrom("utils", "read.csv")
importFrom("lattice", "xyplot")
importFrom("Rdpack", "reprompt")
importFrom("TFisher", "p.tpm", "stat.tpm")
importFrom("qqconf", "qq_conf_plot")
importFrom("mutoss", "ranktruncated")
import("mathjaxr")

An easy-to-use meta-programming layer for Python.

# Motivation

After several thousands of lines of code written in Python, I've come across
several language features which would make my Python programming much more
efficient, but which will probably not appear in standard Python any time soon.
So, I finally decided to implement a tool that allows me to have these features.

`metap` is an easy-to-use meta-programming layer for Python. It allows you to
write programs that generate programs. That sounds fancy, but in practice
`metap` just automates tedious program transformations and programming patterns.

# Installation

First (because `metap` needs a bug-fixed version of `astor`):
```
pip install git+https://github.com/baziotis/astor#egg=astor
```

Then:
```
pip install metap
```


# Quickstart

`metap` works with two scripts: (a) A client, and (b) a meta-program, both
written in Python. This sounds complex but basically your meta-program is just
your program, except it _may_ have `metap`-specific features. The client tells
`metap` how to process your meta-program to generate the actual program.

So, here's a simple example. Let's say you have the following meta-program, in
file `test_mp.py`:

```python,linenos
# test_mp.py
def foo():
  return 2

def bar():
  a = 2

  if a == 2:
    return 4
  
foo()
bar()
```

In this simple example, the meta-program has nothing `metap`-specific. You can
just run it with Python as it is. But, you can still tell a client to transform
it in various useful ways. For example, you may want to log all the `return`s.
So, we write a simple client:

```python
# client.py
from metap import MetaP

mp = MetaP(filename="test_mp.py")
mp.log_returns(include_fname=True)
mp.dump(filename="test.py")
```

This says the minimum `metap` needs to know: which file to load (`test_mp.py`),
what to do (log the returns), and dump the result in `test.py`. Now, we first
run:

```bash
python client.py
```

to produce `test.py`. And then we run it:

```bash
python test.py
```

which produces:

```bash
metap::test_mp.py::Return(ln=3)
metap::test_mp.py::Return(ln=9)
```

So, every time a return fires, we log it. Note that `metap` retained the line
numbers of the _original_ program (i.e., the meta-program), which is what you
want because this is what you're working on. 

`metap` allows you to log all kinds of things, optionally supporting indentation
and only logging within ranges. Here's another simple example of logging when we enter into function bodies:

```python,linenos
# test_mp.py
def bar():
  return 2

def baz():
  return 3

def foo(n):
  if n == 2:
    return foo(3)
  a = bar()
  b = baz()

foo(2)
```

Now, using `mp.log_func_defs(indent=True)` and running the produced `test.py`
we get:

```
metap::FuncDef(ln=7,func=foo)
  metap::FuncDef(ln=7,func=foo)
    metap::FuncDef(ln=1,func=bar)
    metap::FuncDef(ln=4,func=baz)
```

To finish this quickstart guide, things get really interesting when the
meta-program starts using `metap`-specific features.

This example is taken straight from actual code I've written for a
markdown-to-html compiler I use to write [my articles](https://sbaziotis.com/#blog). I want to parse a line and I
want to see if it's a heading, which means it starts with `#`. But, I also care
about whether it's a level-1 heading (i.e., `<h1>`) or level-2 (i.e.,
`<h2>`), to generate the appropriate code. With `metap` I can simply write
the following:

```python
# mdhtml_mp.py
line = "# test"
if (_cvar(line.startswith('# '), hlvl, 1) or
    _cvar(line.startswith('## '), hlvl, 2)):
  print(hlvl)
```

and I use the following client:

```python
from metap import MetaP

mp = MetaP(filename="mdhtml_mp.py")
mp.compile()
mp.dump(filename="mdhtml.py")
```

`mp.compile()` handles _all_ the `metap`-specific features in a single call.
After generating `mdhtml.py` and running it, I get `1`. You can tell how useful
this is by trying to write it in standard Python :)



# Client API

## `class MetaP`

The whole API is under the `MetaP` class. Fields:

- `filename`: The path to the meta-program.

### `MetaP.log_returns()`

**Parameters**:
- `include_fname: str`: Optional. Include the filename in the logs
- `range: List[Union[int, Tuple[int, int]]]`: Optional. Only log returns within the line
  ranges provided. `range` gets a list that can have either integers (denoting a
  single line), or a pair of integers (denoting a `[from, to]` range). 

**Example**

See [Quickstart](#quickstart).


### `log_breaks()` and `log_continues()`

Similar to `log_returns()` but for `break` and `continue`.

**Parameters**:
- `range: List[Union[int, Tuple[int, int]]]`: Optional. Only log returns within the line
  ranges provided. `range` gets a list that can have either integers (denoting a
  single line), or a pair of integers (denoting a `[from, to]` range). 


### `MetaP.log_calls()`

Log call-sites

**Parameters**:
- `range: List[Union[int, Tuple[int, int]]]`: Optional. Only log returns within the line
  ranges provided. `range` gets a list that can have either integers (denoting a
  single line), or a pair of integers (denoting a `[from, to]` range). 

**Example**

```python
# test_mp.py
def add_one(num):
  return num + 1

for x in [0, 1, 2]:
  if x != 0:
    add_one(x)
```

```python
# client.py
import metap
mp = metap.MetaP(filename='test_mp.py')
mp.log_calls()
mp.dump('test.py')
```

Running the generated `test.py`, we get:
```
metap::Call(ln=6,call=add_one(x))
metap::Call(ln=6,call=add_one(x))
```

### `MetaP.log_calls_start_end()`

Prints a message before and after calls matching a pattern.

**Parameters**:
- `patt: Pattern`: Optional. A regular expression. Only function calls that have
  function names that match this pattern are logged.
- `range: List[Union[int, Tuple[int, int]]]`: Optional. Only log returns within the line
  ranges provided. `range` gets a list that can have either integers (denoting a
  single line), or a pair of integers (denoting a `[from, to]` range). 

**Simple Example**

```python
# test_mp.py
with open('d.json', 'w') as fp:
  json.dump(d, fp)
```

```python
import metap
mp = metap.MetaP(filename="test_mp.py")
mp.log_calls_start_end(patt=r'.*json\.dump')
mp.dump(filename="test.py")
```

Running the generated `test.py` gives us:
```
metap: Started executing: 3:json.dump
metap: Finished executing: 3:json.dump
```

### `MetaP.log_func_defs()`

Log when we get into functions.

**Parameters**:
- `range: List[Union[int, Tuple[int, int]]]`: Optional. Only log returns within the line
  ranges provided. `range` gets a list that can have either integers (denoting a
  single line), or a pair of integers (denoting a `[from, to]` range).
- `indent: bool`: Indent the logs such that the indentation is proportional to a call's depth.

**Example**

```python
# test_mp.py
import ast

class RandomVisitor(ast.NodeVisitor):
  def visit_Assign(self, asgn:ast.Assign):
    for t in asgn.targets:
      self.visit(t)
    self.visit(asgn.value)
  
  def visit_BinOp(self, binop:ast.BinOp):
    self.visit(binop.left)
    
code = """
a = b + 2
"""

t = ast.parse(code)
v = RandomVisitor()
v.visit(t)
```

```python
# client.py
import metap
mp = metap.MetaP(filename='test_mp.py')
mp.log_func_defs(indent=True)
mp.dump('test.py')
```

Running the generated `test.py`, we get:
```
metap::FuncDef(ln=4,func=visit_Assign)
  metap::FuncDef(ln=9,func=visit_BinOp)
```


### `MetaP.log_ifs()`

**Parameters**:
- `range: List[Union[int, Tuple[int, int]]]`: Optional. Only log returns within the line
  ranges provided. `range` gets a list that can have either integers (denoting a
  single line), or a pair of integers (denoting a `[from, to]` range). 
- `indent: bool`: Indent the logs such that the indentation is proportional to
  the nesting depth.

**Example**:

```python
# test_mp.py
if True:
  if False:
    pass
  else:
    pass
  
  if True:
    pass
else:
  pass
```

```python
# client.py
import metap
mp = metap.MetaP(filename='test_mp.py')
mp.log_ifs(indent=True, range=[1, (7, 10)])
mp.dump('test.py')
```

Running the generated `test.py`, we get:
```
metap::If(ln=1)
  metap::If(ln=7)
```

Note that the inner `if` with the `else` was not logged because it's not within the ranges.

### `MetaP.dyn_typecheck()`

Adds asserts that verify type annotations in function arguments, returns, and
assignments.

**Parameters**:
- `typedefs_path: str`: Optional. Path to a file with typedefs of the form
  `name = annotation` if the annotations in the main file use anything other than
  the supported names from the `typing` module.
- `skip_funcs: List[str]`: Optional. A list of function names to skip.

Currently supported annotations from `typing`: `Optional`, `Union`, `Tuple`, `List`, `Dict`

**Simple Example**

```python
# test_mp.py
def foo(s: Optional[str]):
  pass
```

```python
# client.py
import metap
mp = metap.MetaP(filename='test_mp.py')
mp.dyn_typecheck()
mp.dump('test.py')
```

The generated `test.py` is:

```python
def foo(s: Optional[str]):
  if not (isinstance(s, str) or s is None):
    print(s)
    print(type(s))
    assert False
  pass
```

**Using Custom Typedefs**

```python
# typedefs.py
TableName = str
ColName = str
ColType = Union[int, float, str]
Col = Tuple[ColName, ColType]
Schema = Dict[TableName, List[Col]]
```

```python
# test_mp.py
def foo(sch: Schema):
  pass
```

```python
# client.py
import metap
mp = metap.MetaP(filename='test_mp.py')
mp.dyn_typecheck()
mp.dump('test.py')
```

### `MetaP.expand_asserts()`

Expands some asserts such that if they fire, you get some info on the expressions involved.

**Parameters**: None

**Simple Example**

```python
a = 2
def foo():
  global a
  a = a + 1
  return a

assert foo() != 3
```

```python
# ...
mp.expand_asserts()
```

The generated `test.py` is:

```python
# ... Same as before
_metap_l = foo()
_metap_r = 3
if _metap_l == _metap_r:
  print(_metap_l)
  print(_metap_r)
  assert False
```

Currently it supports (in)equals (e.g., `assert a == b`) and `isinstance()`
calls (e.g., `assert isinstance(a, int)`).


### `MetaP.dump()`

Generate valid Python code and dump it to a file.

**Parameters**:
- `filename: str`: Optional. If not provided, `metap` will use `<original name>.metap.py`.


# `metap` Superset of Python

All the features we've seen up to now make running a `metap` client _optional_. In other words, you could just run the `test_mp.py` programs without using a client at all.

All the following features extend the Python programming language so using a
`metap` client is mandatory. All these features are handled by
`MetaP.compile()`. So, all the clients in all the following examples are simply:

```python
import metap

mp = metap.MetaP(filename='test_mp.py')
mp.compile()
mp.dump('test.py')
```

### `_ret_ifnn()` and `_ret_ifn()`

**Parameters**:
- `e`: Any expression.

We introduce two new statements that return only under a condition. By far the
two most common conditions I've used in practice are: (1) return `x` if `x` is
not `None` and (2) return `None` if `x` is `None`. Both can be expressed simply
with:

**Example**

```python
# test_mp.py

# Return None if `x` is None
_ret_ifn(x)
# Return `x` if `x` is not None
_ret_ifnn(x)
```

The generated `test.py` is equivalent to:

```python
if x is None:
  return None
if x is not None:
  return x
```

**Usage notes**:

You can use these statements wherever you'd use a return statement. Note that it
_looks_ like a function call but you should think of it as a statement. For
example, the following will _not_ compile:

```python
foo(_ret_ifn(x))
```

Also, note that you can compose this feature with logging returns. For example, you
can issue `mp.compile()`, which will create the `if-return`, and then use
`mp.log_returns()` which will log the generated returns (but using the line
numbers of the original call).

### `cvar()`

**Example**

See [Quickstart](#quickstart).

**Example 2**:

I'll present a slight variation of `_cvar`, where the variable takes the value
of the condition, no matter whether it's true or false.

```python
if _cvar(line.startswith('# '), c):
  # c gets the value True
else:
  # c gets the value False
```

This is basically similar to C++'s:

```c++
if (c = line.startswith("# "))
```

**Usage notes**:

Currently `_cvar()` works only in `if-elif` conditions.

### `time_e()`

Time expression.

**Parameters**:
- `e`: Any expression


**Example**:

```
res, ns = _time_e(2 + 3)
```

`res` gets `5` and `ns` gets the timing in nanoseconds.


### `_mprint()`

Print the expression source along with the expression value.

**Parameters**:
- `e`: Any expression


**Example**:

```
a = 2
_mprint(a)
```

Prints `a: 2`.

# Design Choices

It may seem that it would be better to extend the language, as this e.g., would
allow us to add custom `return` statements that accept a condition, instead of
using call-like syntax with `_ret_ifn()`. However, this option has serious
drawbacks. First, it's a [hefty
task](https://aroberge.github.io/ideas/docs/html/#ideas-making-it-easier-to-extend-python-s-syntax).
But most importantly, it would be hard to reuse the work across environments,
machines, etc., because one would have to have my custom version of Python
everywhere.

On the other hand, meta-programming is portable, easy to reason about (because
you can always see the generated Python code), and opt-in (because you can
choose not to use the `metap` superset of Python).


# Status

`metap` is still in an experimental version, so it should be used with caution
in production. But, it is under active development. Moreover, thankfully `metap`
provides many features that don't _require_ you to run `metap` to get valid
Python. For example, you can use `log_returns()` during debugging and then just
use what you wrote (i.e., the original meta-program, without going through
`metap`) in production.

# Contributing

The most useful contributions at the moment are bug reports and feature requests
(both in the form of [Github issues](https://github.com/baziotis/metap/issues)).
But, pull requests are always welcome.

sumz <-
function(p, weights = NULL, data = NULL, subset = NULL, na.action = na.fail)  {
   if(is.null(data)) data <- sys.frame(sys.parent())
   mf <- match.call()
   mf$data <- NULL
   mf$subset <- NULL
   mf$na.action <- NULL
   mf[[1]] <- as.name("data.frame")
   mf <- eval(mf, data)
   if(!is.null(subset)) mf <- mf[subset,]
   mf <- na.action(mf)
   p <- as.numeric(mf$p)
   weights <- mf$weights
   noweights <- is.null(weights)
   if(noweights) weights <- rep(1, length(p))
   if(length(p) != length(weights)) warning("Length of p and weights differ")
   keep <- (p > 0) & (p < 1)
   invalid <- sum(1L * keep) < 2
   if(invalid) {
      warning("Must have at least two valid p values")
      res <- list(z = NA_real_, p = NA_real_,
         validp = p[keep], weights = weights)
   } else {
      if(sum(1L * keep) != length(p)) {
         warning("Some studies omitted")
         omitw <- weights[!keep]
         if((sum(1L * omitw) > 0) & !noweights)
            warning("Weights omitted too")
      }
      zp <- (qnorm(p[keep], lower.tail = FALSE) %*% weights[keep]) /
         sqrt(sum(weights[keep]^2))
      res <- list(z = zp, p = pnorm(zp, lower.tail = FALSE),
         validp = p[keep], weights = weights)
   }
   class(res) <- c("sumz", "metap")
   res
}
print.sumz <- function(x, ...) {
   cat("sumz = ", x$z, "p = ", x$p, "\n")
   invisible(x)
}

allmetap <- function(p, method = NULL) {
   if(is.null(method)) stop("Must specify a method")
mydata <-
  '"funcs","eponyms"
   "logitp",
   "maximump",
   "meanp",
   "meanz",
   "minimump","Tippett"
   "sumlog","Fisher"
   "sump","Edgington"
   "sumz","Stouffer"
'
   con <- textConnection(mydata)
   details <- read.csv(con, stringsAsFactors = FALSE)
   row.names(details) <- c("logitp", "maximump", "meanp", "meanz",
      "minimump", "sumlog", "sump", "sumz")
   close(con)
# if all specified reset funcnames
funcnames <- method
if("all" %in% method) funcnames <- row.names(details)
# check all funcnames are valid functions
keep <- funcnames %in% row.names(details)
if(sum(keep * 1L) != length(funcnames)) warning("Some unsupported methods specified")
funcnames <- funcnames[keep]
if(length(funcnames) < 1) stop("No supported methods specified")
helper <- function(x) {
   thisfunc <- match.fun(row.names(details[x,]))
   res <- try(thisfunc(p), silent = TRUE)
   if(inherits(res, "try-error")) {
      res <- list(p = NA, valid = NA)
   } else {
      res <- list(p = res$p, valid = length(res$validp))
   }
   res
}
   res <- as.data.frame(t(sapply(funcnames, helper)))
   eponym <- vector("character", nrow(res))
   for(i in 1:nrow(res)) {
      eponym[i] <- details[row.names(res[i,]),"eponyms"]
   }
   res$eponym <- eponym
   class(res) <- c("allmetap", "data.frame")
   res
}
print.allmetap <- function(x, digits = 5, ...) {
   print(format(x, digits = digits, ...), ...)
   invisible(x)
}

sump <-
function(p)  {
   keep <- (p >= 0) & (p <= 1)
   invalid <- sum(1L * keep) < 2
   if(invalid) {
      warning("Must have at least two valid p values")
      res <- list(p = NA_real_, conservativep = NA_real_, validp = p[keep])
   } else {
      sigmap <- sum(p[keep])
      k <- length(p[keep])
      conservativep <- exp( k * log(sigmap) - lgamma(k + 1))
      nterm <- floor(sigmap) + 1 # how many values of sump
      denom <- lfactorial(k)
      psum <- 0
      terms <- vector("numeric", nterm)
      for (i in 1:nterm) {
         terms[i] <- lchoose(k, i - 1) + k * log(sigmap - i + 1) - denom
         pm <- 2 * (i %% 2) - 1
         psum <- psum + pm * exp(terms[i])
      }
      if(k != length(p)) {
         warning("Some studies omitted")
      }
      if(sigmap > 20) {
         warning("Likely to be unreliable, check with another method")
      }
      res <- list(p = psum, conservativep = conservativep, validp = p[keep])
   }
   class(res) <- c("sump", "metap")
   res
}
print.sump <- function(x, ...) {
   cat("psum = ", x$p, "\n")
   invisible(x)
}

two2one <-
function(p, two = NULL, invert = NULL) {
   np <- length(p)
   if(is.null(two)) {
      two <- rep(TRUE, np)
   }
   if(is.null(invert)) {
      invert <- rep(FALSE, np)
   }
# check values of p
   inrange <- sum(1L * ((p >= 0) & (p <= 1)))
   if(np != inrange)
      warning("Some p out of range")
   onep <- ifelse(two,
      ifelse(invert, (1 - p) + p / 2, p / 2),
      ifelse(invert, 1 - p, p)
   )
   onep
}

meanz <-
function(p)  {
   keep <- (p > 0) & (p < 1)
   invalid <- sum(1L * keep) < 2
   if(invalid) {
      warning("Must have at least two valid p values")
      res <- list(z = NA_real_, p = NA_real_,
         validp = p[keep])
   } else {
      if(sum(1L * keep) != length(p)) {
         warning("Some studies omitted")
      }
      zvals <- (qnorm(p[keep], lower.tail = FALSE))
      zp <- mean(zvals) / (sd(zvals) / sqrt(length(p[keep])))
      res <- list(z = zp, p = pnorm(zp, lower.tail = FALSE),
         validp = p[keep])
   }
   class(res) <- c("meanz", "metap")
   res
}
print.meanz <- function(x, ...) {
   cat("meanz = ", x$z, "p = ", x$p, "\n")
   invisible(x)
}

sumlog <-
function(p) {
   keep <- (p > 0) & (p <= 1)
   invalid <- sum(1L * keep) < 2
   if(invalid) {
      warning("Must have at least two valid p values")
      res <- list(chisq = NA_real_, df = NA_integer_,
         p = NA_real_, validp = p[keep])
   } else {
      lnp <- log(p[keep])
      chisq <- (-2) * sum(lnp)
      df <- 2 * length(lnp)
      if(length(lnp) != length(p)) {
         warning("Some studies omitted")
      }
      res <- list(chisq = chisq, df = df,
         p = pchisq(chisq, df, lower.tail = FALSE), validp = p[keep])
    }
   class(res) <- c("sumlog", "metap")
   res
}
print.sumlog <- function(x, ...) {
   cat("chisq = ", x$chisq, " with df = ", x$df, " p = ", x$p, "\n")
   invisible(x)
}

plotp <- function(pvals, plotversion = "qqconf", ...) {
   keep <- !is.na(pvals)
   validp <- pvals[keep] # remove NA
   keep <- validp >= 0 & validp <= 1
   if(sum(1L * keep) < 1) stop("No points to plot")
   n <- length(validp)
   if(sum(1L * keep) != n) warning("Out of range points omitted")
   y <- validp[keep]
   n <= length(y)
   if(plotversion == "qqconf") {
      if(requireNamespace("qqconf")) {
         qqconf::qq_conf_plot(y, distribution = qunif, ...)
      } else {
         warning("qqconf not installed, consider setting plotversion to old")
      }
   } else {
      qqplot(qunif(ppoints(n)), y, xlab = "Theoretical",
         ylab = "Empirical", ...)
      qqline(y, distribution = qunif, ...)
   }
   res <- list(validp = y)
   invisible(res)
}
plot.metap <- function(x, plotversion = "qqconf", ...) {
   plotp(x$validp, plotversion = plotversion, ...)
   invisible(x)
}

votep <-
function(p, alpha = c(0.5, 0.5)) {
   alpha <- ifelse(alpha > 1, alpha / 100, alpha) # if percent
   stopifnot(alpha > 0, alpha < 1)
   stopifnot(alpha[1] <= alpha[2])
   keep <- (p >= 0) & (p <= 1)
   invalid <- sum(1L * keep) < 2
   if(invalid) {
      warning("Must have at least two valid p values")
      res = list(p = NA_real_, pos = NA_integer_, neg = NA_integer_,
         alpha = alpha, validp = p[keep])
   } else {
      pi <- p[keep]
      k <- length(pi)
      pos <- sum(1L * (pi < alpha[1]))
      neg <- sum(1L * (pi > alpha[2]))
      if(k != length(p)) {
         warning("Some studies omitted")
      }
      if((pos + neg) <= 0) {
         warning("All p values are within specified limits of alpha")
         p <- 1
      } else {
         p = binom.test(pos, pos + neg, 0.5, alternative = "greater")$p.value
      }
      res = list(p = p, pos = pos, neg = neg, alpha = alpha, validp = pi)
   }
   class(res) <- c("votep", "metap")
   res
}
print.votep <- function(x, ...) {
   cat("p = ", x$p, "\n")
   invisible(x)
}

logitp <-
function(p)  {
   keep <- (p > 0) & (p < 1)
   invalid <- sum(1L * keep) < 2
   if(invalid) {
      warning("Must have at least two valid p values")
      res <- list(t = NA_real_, df = NA_integer_, p = NA_real_,
         validp = p[keep])
   } else {
      psum <- sum(log(p[keep] / (1 - p[keep])))
      k <- length(p[keep])
      mult <- -1 / sqrt(k * pi ^ 2 * (5 * k + 2) / (3 * (5 * k + 4)))
      if(k != length(p)) {
         warning("Some studies omitted")
      }
      t <- mult * psum
      df <- (5 * k + 4)
      res <- list(t = t, df = df, p = pt(t, df, lower.tail = FALSE),
         validp = p[keep])
   }
   class(res) <- c("logitp", "metap")
   res
}
print.logitp <- function(x, ...) {
   cat("t = ", x$t, " with df = ", x$df, " p = ", x$p, "\n")
   invisible(x)
}

schweder <- function(p, xlab = "Rank of p", ylab = "p",
   drawline = NULL,
#   bh.control =list(),
   bh.lwd = 1, bh.lty = "solid", bh.col = "black",
   ls.control =list(frac = NULL),
   ls.lwd = 1, ls.lty = "dotted", ls.col = "black",
   ab.control =list(a = NULL, b = NULL),
   ab.lwd = 1, ab.lty = "dashed", ab.col = "black",
    ...) {
   bx1y1toa <- function(b, x1, y1) { # find intercept from b, x1, y1
      a <- y1 - b * x1
      a
   }
   bh <- function(p) { # parameters for Benjamini and Hochberg lowet slope line
      n <- length(p)
      rankp <- n:1
      si <- (1 - p) / (n - rankp + 1) # assumes p in non-ascending order
      di <- which(diff(si) > 0)
      ldi <- max(1, di[length(di)]) # in case no slope change
      b <- si[ldi]
      res <- list(a = bx1y1toa(b, n + 1, 1), b = b, si = si, di = di, ldi = ldi)
      res
   }
   n <- length(p)
   stopifnot(n > 1)
   toplot <- rev(sort(p))
   keep <- (toplot >= 0) & (toplot <= 1)
   toplot <- toplot[keep]
   if(length(toplot) != n) warning('Some p values omitted')
   n <- length(toplot)
   if(n < 1) stop("Not even one to plot")
   x <- n:1
   plot(x, toplot, ylab = ylab, xlab = xlab,
      ylim = c(0, 1), xlim = c(0, n+1), ...)
   res <- list(p = toplot)
   if("bh" %in% drawline) {
      params <- bh(toplot)
      abline(a = params$a, b = params$b, lty = bh.lty,
          lwd = bh.lwd, col = bh.col)
      res <- c(res, bh.params = list(a = params$a, b = params$b,
         si = params$si, di = params$di, ldi = params$ldi))
   }
   if("ls" %in% drawline) {
      if(is.null(ls.control$frac)) {
         warning("Must specify fraction for ls")
      } else {
         frac <- ls.control$frac
         newy <- toplot - 1  # transform so that line is forced
         newx <- x - (n + 1) # through (n+1),1 rather than 0,0
         plim <- newy[n * frac]
         fit <- lm(newy ~ newx - 1, subset = newy > plim)
         b <- coef(fit)[1] # correct slope
         a <- bx1y1toa(b, (n + 1), 1) # passing through (n+1,1
         abline(a = a, b = b, lty = ls.lty,
            lwd = ls.lwd, col = ls.col)
         res <- c(res, ls.params = list(a = a, b = b))
      }
   }
   if ("ab" %in% drawline) {
      if( is.null(ab.control$a) | is.null(ab.control$b) ) {
         warning("No parameters for a and b line")
      } else {
         abline(a = ab.control$a, b = ab.control$b,lty = ab.lty,
            lwd = ab.lwd, col = ab.col)
         res <- c(res, ab.params = list(a = ab.control$a, b = ab.control$b))
      }
   }
   invisible(res)
}

truncated <- function(p, ptrunc = NULL, rtrunc = NULL, ...) {
#
dop <- function(validp, ptrunc) {
   if(requireNamespace("TFisher")) {
      if((ptrunc <= 0) | (ptrunc > 1))
      {
         warning("The value of ptrunc must be between 0 and 1")
         warning("Setting p to 0.5")
         ptrunc <- 0.5
      }
      nincl <- sum(1L * (validp < ptrunc))
      nexcl <- sum(1L * (validp >= ptrunc))
      if(nincl < 1) warning("No p values left after truncation")
      if(nexcl < 1) message("No studies removed by truncation")
      stat <- TFisher::stat.tpm(validp, ptrunc)
      pval <- 1 - TFisher::p.tpm(stat, length(validp), ptrunc)
      res <- list(p = pval, stat = stat, validp = validp,
         ptrunc = ptrunc, n = c(incl = nincl, excl = nexcl),
         method = "truncated")
   } else {
      warning("Need to install TFisher for this option")
      res <- list(p = NA, stat = NA)
   }
   res
}
#
dor <- function(validp, rtrunc, ...) {
   if(requireNamespace("mutoss")) {
      if((rtrunc < 1) | (rtrunc > length(validp)))
      {
         l <- length(validp)
         warning(paste("The value of rtrunc must be between 1 and "), l)
         warning(paste("Setting rtrunc to", l %/% 2))
         rtrunc <- l %/% 2
      }
      nincl <- rtrunc
      nexcl <- length(validp) - nincl
      if(nincl < 1) warning("No p values left after truncation")
      if(nexcl < 1) message("No studies removed by truncation")
      temp <- mutoss::ranktruncated(validp, rtrunc, ...)
      stat <- temp$RTP$Statistic
      pval <- temp$RTP$p.Value
      res <- list(p = pval, stat = stat, validp = validp,
         rtrunc = rtrunc, n = c(incl = nincl, excl = nexcl),
         method = "rank")
      } else {
         warning("Need to install mutoss for this option")
         res <- list(p = NA, stat = NA)
      }
   res
}
#
   keep <- (p > 0) & (p <= 1)
   if(sum(1L * keep) < 2) {
      warning("Must have at least two valid p values")
      res <- list(p = NA, stat = NA)
   } else {
      validp <- p[keep]
      if(length(validp) != length(p)) {
         warning("Some studies omitted")
      }
      usetpm <- !is.null(ptrunc)
      userank <- !is.null(rtrunc)
      if(usetpm & userank) {
         warning("Specifying both ptrunc and rtrunc undefined")
         res <- list(p = NA, stat = NA)
      } else if(usetpm) {
         res <- dop(validp, ptrunc)
      } else if(userank) {
         res <- dor(validp, rtrunc)
      } else {
         warning("Must specify one of ptrunc or rtrunc")
         res <- list(p = NA, stat = NA)
      }
   }
   class(res) <- c("truncated", "list")
   res
}
print.truncated <- function(x, ...) {
   cat("stat = ", x$stat, " p = ", x$p, "\n")
   invisible(x)
}
summary.truncated <- function(object, ...) {
   with(object, {
      cat("stat = ", stat, " p = ", p, "\n")
      if(!is.na(stat)) {
         cat(paste("Using", n["incl"], "values of",
            n["incl"] + n["excl"], "valid p values\n"))
         if(method == "truncated") {
            cat("Truncated at p= ", ptrunc, "\n")
         }
         if(method == "rank") {
            cat("Truncated at rank= ", rtrunc, "\n")
         }
      } # end valid stat
   }) # end with
   invisible(object)
}
plot.truncated <- function(x, pparams = list(pchs = c(16, 1),
      pcols = c("black", "black")), ...) {
   if(is.na(x$stat)) {
      warning("Nothing to plot")
   } else {
      with(x, {
         p <- sort(validp, decreasing = FALSE)
         nused <- n["incl"]
         k <- length(p)
         plot(1:k, c(rep(0, k-1), 1), type = "n", xlab = "Index",
	        ylab = "p", ...)
         if(nused == k) {
            points(1:nused, p[1:nused], pch = pparams$pchs[1],
               col = pparams$pcols[1])
         } else if(nused == 0) {
            points((nused+1):k, p[(nused+1):k], pch = pparams$pchs[2],
               col = pparams$pcols[2])
         } else {
            points(1:nused, p[1:nused], pch = pparams$pchs[1],
               col = pparams$pcols[1])
            points((nused+1):k, p[(nused+1):k], pch = pparams$pchs[2],
               col = pparams$pcols[2])
         }
      })
   }
   invisible(x)
}

invt <-
function(p, k, data = NULL, subset = NULL, na.action = na.fail) {
   if(is.null(data)) data <- sys.frame(sys.parent())
   mf <- match.call()
   mf$data <- NULL
   mf$subset <- NULL
   mf$na.action <- NULL
   mf[[1]] <- as.name("data.frame")
   mf <- eval(mf, data)
   if(!is.null(subset)) mf <- mf[subset,]
   mf <- na.action(mf)
   p <- as.numeric(mf$p)
   k <- as.numeric(mf$k)
   if(length(k) == 1) {
      k <- rep(k, length(p))
   } else {
   }
   keep <- (p > 0) & (p <= 1) & (k > 2)
   invalid <- sum(1L * keep) < 2
   if(invalid) {
      warning("Must have at least two valid p values")
      res <- list(z = NA_real_, p = NA_real_, validp = p[keep])
   } else {
      pvals <- p[keep]
      k <- k[keep]
      if(length(pvals) != length(p)) {
         warning("Some studies omitted")
      }
      z <- sum(qt(pvals, k, lower.tail = FALSE)) /
         sqrt(sum(k / (k - 2)))
      res <- list(z = z,
         p = pnorm(z, lower.tail = FALSE), validp = pvals)
   }
   class(res) <- c("invt", "metap")
   res
}
print.invt <- function(x, ...) {
   cat("z = ", x$z, " p = ", x$p, "\n")
   invisible(x)
}

wilkinsonp <-
function(p, r = 1, alpha = 0.05) {
   alpha <- ifelse(alpha > 1, alpha / 100, alpha) # if percent
   stopifnot(alpha > 0, alpha < 1)
   alpha <- ifelse(alpha > 0.5, 1 - alpha, alpha) # map to 0 to 0.5
   keep <- (p >= 0) & (p <= 1)
   invalid <- sum(1L * keep) < 2
   if (invalid)   {
      warning("Must have at least two valid p values")
      res <- list(p = NA_real_, pr = NA_real_,
         r = r, critp = NA_real_, alpha = alpha, validp = p[keep])
   } else {
      pi <- p[keep]
      k <- length(pi)
      if(k != length(p)) {
         warning("Some studies omitted")
      }
      if((r < 1) | (r > k)) {
         r <- 1
         warning("Illegal r set to 1")
      }
      pi <- sort(pi)
      pr <- pi[r]
      res <- list(p = pbeta(pr, r, k + 1 - r), pr = pr,
         r = r, critp = qbeta(alpha, r, k + 1 - r), alpha = alpha, validp = pi)
    }
   class(res) <- c("wilkinsonp", "metap")
   res
}
maximump <- function(p,  alpha = 0.05) {
   keep <- (p >= 0) & (p <= 1)
   validp <- p[keep]
   k <- length(validp)
   res <- wilkinsonp(p, r = k, alpha)
   class(res) <- c("maximump", "metap")
   res
}
minimump <- function(p,  alpha = 0.05) {
   res <- wilkinsonp(p, r = 1, alpha)
   class(res) <- c("minimump", "metap")
   res
}
print.wilkinsonp <- function(x, ...) {
   r <- x$r
   p <- x$p
   cat("p = ", p, " using ", paste(r, "th minimum p", sep = ""), "\n")
   invisible(x)
}
print.maximump <- function(x, ...) {
   p <- x$p
   cat("p = ", p, " using maximum p", "\n")
   invisible(x)
}
print.minimump <- function(x, ...) {
   p <- x$p
   cat("p = ", p, " using minimum p", "\n")
   invisible(x)
}

albatros <- function(p, n, axes = list(xlimit = NULL, ylimit = NULL,
      lefttext = NULL, righttext = NULL, xlas = 3, ylas = 2),
   contours = list(type = NULL, contvals = NULL, ltys = NULL, contlabs = NULL),
   plotpars = list(yscale = "sqrt", yfun = NULL, pchs = NULL, cols = NULL), ...) {
#
# define axis transformation functions
xtrans <- makexscale(p)
ytrans <- makeyscale(plotpars$yscale, plotpars$yfun) 
#
   if(length(p) != length(n)) {
      stop(paste("p", length(p), " and n", length(n), "must be same length"))
   }
   keep <- (p < 1) & (p > 0)
   validp <- p[keep] # remove zeroes and ones for axis calc and plotting
   validn <- n[keep] 
   is.na(validn) <- n < 2 # need to remove illegal n as well
   extreme <- min(min(validp), min(1 - validp))
# set up axes
   if(is.null(axes$xlimit)) { # set limits for x
      xlims <- c(-xtrans(extreme), xtrans(extreme))
   } else {
      xlims <- c(-xtrans(axes$xlimit), xtrans(axes$xlimit))
   }
   if(is.null(axes$ylimit)) { # set limit for y allowing space
      ylims <- c(ytrans(min(validn, na.rm = TRUE)) * 0.9,
         ytrans(max(validn, na.rm = TRUE)) * 1.1)
   } else {
      ylims <- c(ytrans(axes$ylimit[1]), ytrans(axes$ylimit[2]))
   }
#   
   if(is.null(plotpars$pchs)) {
      pchs <- rep(1, length(validp))
   } else {
      pchs <- plotpars$pchs
   }
   if(is.null(plotpars$cols)) {
      cols <- rep("lightgrey", length(validp))
   } else {
      cols <- plotpars$cols
   }
   # now plot points without any axes
   plot(xtrans(validp), ytrans(validn), 
     xlim = xlims, ylim = ylims, pch = pchs,
     xaxt = "n", xlab = "p value", yaxt = "n", ylab = "N", ...
   )
#
# set up x-axis values and plot axis
   sigs <- c(0.05, 0.01)
   i <- 2
   while(sigs[i] > extreme) { # include 0.001, 0.0001 as necessary
     sigs[i+1] <- sigs[i] / 10
     i <- i + 1
   }
   axis(1, at = c(-xtrans(sigs), 0, xtrans(sigs)),
      labels = c(sigs, "null", sigs), cex.axis = 0.75,
         las = axes$xlas, ...)
# set up y-axis values and plot axis
   maxn <- max(validn, na.rm = TRUE)
   if(maxn <= 500) {
      ylabs <- c(1, 2, 5)
      i <- 3
      while(ylabs[i] < maxn) { # include 10, 20, 50 as necessary
         ylabs[i+1] <- ylabs[i - 2] * 10
         i <- i + 1
      }
   } else {
      ylabs <- c(1, 5)
      i <- 2
      while(ylabs[i] < maxn) { # include 10, 50, 100 as necessaryw
         ylabs[i + 1] <- ylabs[i - 1] * 10
         i <- i + 1
      }
   }
   axis(2, at = ytrans(ylabs), labels = ylabs, cex.axis = 0.75,
      las = axes$ylas, ...)
# plot contours
   docontours(contours$type, sigs, contours$contvals, xtrans, ytrans,
      xlims, ylims, contours$ltys, contours$contlabs)
   if(!is.null(axes$lefttext)) {
      mtext(axes$lefttext, side = 1, line = 3, adj = 0)
   }
   if(!is.null(axes$righttext)) {
      mtext(axes$righttext, side = 1, line = 3, adj = 1)
   }
   # what to return here as well?
   res <- list(xlims = xlims, ylims = ylims, validp = validp,
      validn = validn)
   res
}

meanp <-
function(p) {
   keep <- (p >= 0) & (p <= 1)
   invalid <- sum(1L * keep) < 4
   if(invalid) {
      warning("Must have at least four valid p values")
      res <- list(z = NA_real_, p = NA_real_,
         validp = p[keep])
   } else {
      pi <- mean(p[keep])
      k <- length(p[keep])
      z <- (0.5 - pi) * sqrt(12 * k)
      if(k != length(p)) {
         warning("Some studies omitted")
      }
      res <- list(z = z, p = pnorm(z, lower.tail = FALSE),
         validp = p[keep])
   }
   class(res) <- c("meanp", "metap")
   res
}
print.meanp <- function(x, ...) {
   cat("z = ", x$z, " p = ", x$p, "\n")
   invisible(x)
}

docontours <- function(conttype, xvals, contvals, xtrans, ytrans,
   xlims, ylims, ltys, contlabs) {
#
# conttype Character: one of "smd", "or", "corr"}
# xvals A vector of values to evaluate the contour}
# contvals The values at which contours are drawn}
# xtrans A function for transforming the x-axis}
# ytrans A function for transforming the y-axis}
# xlims The limits for the x-axis in plotting units}
# ylims The limits for the y-axis in plottng units}
# ltys A vector of line types for the contours}
# contlabs Logical, draw the labels of the contours?}
   corr2n <- function(p, r) {
      res <- qnorm(p) ^ 2 / atanh(r) ^ 2 + 3
      res
   }
#
   smd2n <- function(p, smd) {
      res <- qnorm(p) ^ 2 * (8 + smd ^2) / (2 * smd ^ 2)
      res 
   }
#
   or2n <- function(p, or) {
      lnor2 <- log(or) ^ 2
      res <- qnorm(p) ^ 2 * (8 * pi ^ 2 + 3 * lnor2) / (6 * lnor2)
      res
   }
#
   valid <- TRUE
   if(is.null(conttype)) {
      valid <- FALSE
   } else if(is.null(contvals)) {
      valid <- FALSE
   } else if(!conttype %in% c("smd", "or", "corr")) {
      valid <- FALSE
      warning(paste("Unrecognised effect type", conttype))
   } else if(conttype == "smd") {
      contfunc <- smd2n # set contfunc here as function for contours
   } else if(conttype == "corr") {
      contfunc <- corr2n
   } else if(conttype == "or") {
      contfunc <- or2n
   }
   if(valid) {
      pvals <- c(0.4, xvals)
      # now set up line types
      l <- length(contvals)
      if(is.null(ltys) | length(ltys != l)) {
         ltys <- 1:l
      } else if(length(ltys) == 1) {
         ltys <- rep(ltys[1], l)
      }
      locallabs <- TRUE
      if(!is.null(contlabs)) locallabs <- contlabs
      for(i in 1:l) {
         nvals <- contfunc(pvals, contvals[i])
         temp <- spline(xtrans(pvals), ytrans(nvals), n = 101)
         # now find where it leaves plotting area
         len <- length(temp$x)
         if(locallabs) {
            j <- 1
            while(temp$x[j] < xlims[2] & temp$y[j] < ylims[2] & j < len) {
               j <- j + 1
            }
            textx <- temp$x[j]
            texty <- temp$y[j]
            text(textx, texty, as.character(contvals[i]), pos = 2)
        } 
        # shall we move this out of the function
         lines(temp, lty = ltys[i])
         lines(spline(-xtrans(pvals), ytrans(nvals), n = 101), lty = ltys[i])
      }
      # shall we return anything here including when !valid
   }
}

makexscale <- function(p) {
# out of range p removed by caller
   xtrans <- function(p) {
      p2 <- ifelse(p > 0.5, 1 - p, p)
      res <- log(1 / p2, base = 10)
      res <- ifelse(p > 0.5, -res, res)
      res
   }
   xtrans
}
makeyscale <-
function(yscale, userfun)  {
   nfunc <- function(n) n
   classicfunc <- function(n) {
      (log(n, base = 10))^2
   }
   if(is.null(yscale)) {
      yscale <- "sqrt"
      warning("No yscale specified: defaulting to sqrt")
   }
   if(yscale == "classic") {
      res <- classicfunc
   } else if(yscale == "n") {
      res <- nfunc
   } else if(yscale == "sqrt") {
      res <- sqrt
   } else if(yscale == "userfun") {
      if(is.null(userfun)) {
         warning("You need to specify a function for userfun")
         warning("Defaulting to sqrt")
         res <- sqrt
      } else {
         res <- userfun
      }
   } else {
      warning("Unrecognised y scale, defaulted to sqrt")
      res <- sqrt
   }
   res
}

\documentclass[12pt]{article}
\usepackage{amsmath,amssymb}
%\usepackage{mydef2}
\usepackage[round]{natbib}
\usepackage{parskip,url}
\usepackage{graphicx,subfig}
\setlength{\topmargin}{0cm}
\addtolength{\textheight}{2cm}
\renewcommand{\topfraction}{0.85}
\renewcommand{\bottomfraction}{0.5}
\renewcommand{\textfraction}{0.15}
\renewcommand{\floatpagefraction}{0.8}
%\lhead{}
%\input{title}
%\VignetteIndexEntry{Plotting in the metap package}
\title{Plotting in the \pkg{metap} package}
\author{Michael Dewey}
\newcommand{\pkg}[1]{\texttt{#1}}
\newcommand{\func}[1]{\texttt{#1}}
\newcommand{\code}[1]{\texttt{#1}}
\newcommand{\codefont}{\footnotesize}
\newcommand{\mygraph}[3]{%
\begin{figure}[htbp]
\includegraphics[height=6cm,width=10cm]{#1}
\caption{#2}
\label{#3}
\end{figure}
}
\newcommand{\twograph}[8]{%
\begin{figure}[htbp]
\subfloat[#2\label{#3}]{\includegraphics[height=6cm,width=7cm]{#1}}%
\subfloat[#5\label{#6}]{\includegraphics[height=6cm,width=7cm]{#4}}
\caption{#7}
\label{#8}
\end{figure}
}
\begin{document}
\maketitle

\section{Introduction}

\subsection{What is this document for?}

This document describes how and why to plot
$p$--values in the \pkg{metap} package.
Examining the $p$--values
graphically or otherwise before subjecting them to further analysis
is useful to provide a visual impression
of their distribution and to check for excess $p$--values at
both extremes..
Three functions are provided for this purpose:
\func{albatros}, \func{plotp},
and \func{schweder}.


\subsection{Example datasets}
As our example we use various data-sets:
\begin{description}
\item[\func{teachexpect}]
Effect of teacher expectations on student IQ \citep{becker94}
\item[\code{validity}]
The validity of
student ratings of their instructors \citep{becker94}.
\item[\code{zhang}]
The effect of the timing of exercise interventions
for patients with cardiovascular disease \citep{zhang16}
\end{description}

{\codefont
<<>>=
library(metap)
data(dat.metap)
teach <- dat.metap$teachexpect
validity <- dat.metap$validity$p
zhang <- dat.metap$zhang
print(validity)
@
}

\section{Plotting using \func{plotp}}

The \func{plotp} provides a Q--Q plot of
the $p$--values to detect departure from
the uniform distribution.

<<fig=TRUE,label=plotp,include=FALSE>>=
plotp(validity, main = "Validity data")
@

\twograph{plotmetap-plotp}{Q--Q plot from \func{plotp}}{plotp}{plotmetap-plotfunc}{Legacy Q--Q plot}{plotfunc}{Plots of validity data}{plotvalid}
%\mygraph{plotmetap-plotp}{Q--Q plot from \func{plotp}}{plotp}
%\mygraph{plotmetap-plotfunc}{Q--Q plot an object of class \func{metap}}{plotfunc}

Figure \ref{plotp} shows the resulting plot.
The line represents a fit to the uniform
distribution and the polygon is a simultaneous
confidence region such that if any point lies
outside it we reject the null hypothesis
that the points are drawn iid from a uniform.
Small $p$--values are to the left of the plot

The format of plot shown in Figure \ref{plotp} was first introduced
in version 1.8 of \func{metap}.
The previous plotting function is still available
and it is possible to produce this plot by setting
the \func{plotversion} parameter to "old" in the call
to \func{plotp}.
An example is shown in Figure \ref{plotfunc}
which first calls \func{sumlog}.
The
legacy one will always remain an option.

{\codefont
<<fig=TRUE,label=plotfunc,include=FALSE>>=
plotp(validity, main = "Validity data", plotversion = "old")
@
}

Note that the \func{plot} method for objects
of class \code{"metap"} uses the new version
of the plot. This change was introduced in version 1.9
of this package.

There are many possible options which can be passed to
the plotting function and hence
to the \func{qqconf} plotting
routine.
The documentation for
the \func{qqconf} package should be consulted for details.
The \func{qqconf} package vignette is also very helpful.
We will look at one of those options here though.

{\codefont
<<fig=TRUE,label=teachlinear,include=FALSE>>=
plotp(teach)
@
}
{\codefont
<<fig=TRUE,label=teachlog,include=FALSE>>=
plotp(teach, log10 = TRUE)
@
}
\twograph{plotmetap-teachlinear}{Linear scaling}{teachlinear}{plotmetap-teachlog}{Log scaling}{teachlog}{Teacher expectancy data}{teach}

Figure \ref{teach} shows the teacher expectancy data using the default
scaling in sub--figure \subref{teachlinear}.
It is hard to see whether some of the points fall
outside the boundary.
However if we use the log--scaling option shown in sub--figure
\subref{teachlog} it becomes much clearer.
Note that the scale is reversed between the
sub--plots and in \subref{teachlog}
the small $p$--values are now on the right.
So the cluster of points near the bottom left of the
sub--figure \subref{teachlinear} are hard to distinguish
as to whether they lie inside the boundary or not.
In the log scaling of sub--figure \subref{teachlog}
where they appear towards the top right
it is much clearer that one does
fall outside the boundary and two others are
borderline.
This reflects the fact that for most of the methods in the \func{metap} package
the overall $p$--value is below 0.05.
For instance using the logit method we have

{\codefont
<<>>=
logitp(teach)
@
}

\section{Plotting using \func{schweder}}

A function \func{schweder} provides plots
with a variety of informative lines
superimposed.
It plots the ordered $p$--values, $p_{[i]}: p_{[1]} \le \dots{}
p_{[2]} \le \dots{} p_{[i]} \dots{}
\le p_{[k-1]} \le p_{[k]}$, against
$i$.
Although the original motivation
for the plot is \citet{schweder82}
the function uses a different choice of axes due
to
\citet{benjamini00}.
We will use an example dataset on the validity of student
ratings quoted in \citet{becker94}.
Figure \ref{simple} shows the plot from \func{schweder}.

<<fig=TRUE,label=simple,include=FALSE>>=
schweder(validity)
@

\func{schweder} also offers the possibility of drawing one of 
a number of straight line summaries.
The three possible straight line summaries are
shown in Figure \ref{withlines} and are:

\begin{itemize}
\item
the lowest slope line of Benjaimin and Hochberg
which is drawn by default
as solid,
\item
a least squares line drawn passing through the point
$k+1, 1$
and using a specified fraction of the points
which is drawn by default as dotted,
\item
a line with user specified
intercept and slope
which is drawn by default as dashed.
\end{itemize}

<<fig=TRUE,label=withlines,include=FALSE>>=
schweder(validity, drawline = c("bh", "ls", "ab"),
   ls.control = list(frac = 0.5), ab.control = list(a = 0, b = 0.01))
@
\twograph{plotmetap-simple}{Simple graph}{simple}{plotmetap-withlines}{With lines}{withlines}{Output from schweder}{schweder}

\section{The albatros plot}

The albatros plot was introduced in \citet{harrison17}
which should be consulted for more details.
Basically it consists of plotting a possibly transformed sample
size against the transformed $p$--values.
The default is to use $\sqrt{N}$ for the
$y$--axis and a log transformation for the $x$--axis.
The scale for the y-axis is user selectable.
The original scale in the Stata version is
$(\log_{10}n)^2$ which is obtained by
setting \code{yscale} to "classic".
For small N the default of "sqrt" is very
similar to "classic".
The plot also contains contours of constant
effect size.
A number of possible options are available for effect size
type: correlation,
standardised mean difference, and odds ratio.

<<fig=TRUE,label=albatros,include=FALSE>>=
validity <- dat.metap$validity
fit.v <- albatros(validity$p, validity$n,
   contours = list(type = "corr", contvals = c(0.25, 0.5, 0.8), ltys = 1:3),
      axes = list(ylimit = c(1,200),  lefttext = "Negative correlation",
         righttext = "Positive correlation"),
   main = "Validity")
@

\mygraph{plotmetap-albatros}{Albatros plot from of the validity data}{albatros}

Figure \ref{albatros} shows the result.
Most of the points clearly correspond to positive
and substantial correlations although a
few are in the
opposite direction although not far from the null
$p$--value (0.5).

Of course if the actual effect sizes are available it would be
better to use one of the conventional methods for
meta--analysing them.
\citet{harrison17} outline possible
use cases for this method even so.

If the studies come from different groups one might
use meta--regression with a moderator for group
membership if one had the effect sizes.
In the absence of effect sizes
the albatros plot can display the points using different
symbols for groups.
This would enable a visual check on whether the groups
differed.

<<fig=TRUE,label=zhang,include=FALSE>>=
data(dat.metap)
zhang <- dat.metap$zhang
fit.z <- albatros(zhang$p, zhang$n,
   contours = list(type = "smd", contvals = c(0.25, 0.5, 1), ltys = 1:3),
   plotpars = list(pchs = letters[unclass(zhang$phase)]),
   axes = list(lefttext = "Favours control", righttext = "Favours exercise"),
   main = "Zhang"
)
legend(-4, 11, c("Acute", "Healing", "Healed"),
   pch = c("a", "b", "c"), bg = "white"
)
@

\mygraph{plotmetap-zhang}{Albatros plot of the Zhang et al data}{zhang}

Figure \ref{zhang} shows an example using the Zhang et al data-set.
The studies involved come from three groups
corresponding to three different periods of
initiation of exercise.
The points
are labelled accordingly:
"a" initiation during the acute phase, "b"
during the healing phase and "c" during the healed phase.
The difference between the groups is quite clear here.
In fact in \citet{zhang16} the results are handled with
stratification into three separate analyses
and meta--regression was not used.

If some studies had given effect sizes but others did not then
an albatros plot with the points marked for group
membership and with appropriate contour lines
would provide a visual check on whether the unavailable
effect sizes were similar to the available ones.

\bibliography{metap}
\bibliographystyle{plainnat}
\end{document}

\documentclass[12pt]{article}
\usepackage{amsmath,amssymb}
%\usepackage{mydef2}
\usepackage[round]{natbib}
\usepackage{parskip,url}
\usepackage{graphicx,subfig}
\setlength{\topmargin}{0cm}
\addtolength{\textheight}{2cm}
%\lhead{}
%\input{title}
%\VignetteIndexEntry{Comparison of methods in the metap package}
\title{Comparison of methods in the \pkg{metap} package}
\author{Michael Dewey}
\newcommand{\pkg}[1]{\texttt{#1}}
\newcommand{\func}[1]{\texttt{#1}}
\newcommand{\code}[1]{\texttt{#1}}
\newcommand{\codefont}{\footnotesize}
\newcommand{\mygraph}[3]{%
\begin{figure}[htbp]
\includegraphics[height=6cm,width=10cm]{compare-#1}
\caption{#2}
\label{#3}
\end{figure}
}
% in mypdf fourth parameter needs []
\newcommand{\mypdf}[4]{%
\begin{figure}[htbp]
\includegraphics#4{#1.pdf}
\caption{#2}
\label{#3}
\end{figure}
}
\newcommand{\twograph}[8]{%
\begin{figure}[htbp]
\subfloat[#2\label{#3}]{\includegraphics[height=6cm,width=7cm]{compare-#1}}%
\subfloat[#5\label{#6}]{\includegraphics[height=6cm,width=7cm]{compare-#4}}
\caption{#7}
\label{#8}
\end{figure}
}
\begin{document}
\maketitle

\section{Introduction}

\subsection{What is this document for?}

This document describes some methods for the meta--analysis of
$p$--values
(significance values)
contained in the package \pkg{metap}
and contains comments on
the performance of the various algorithms
under a small number of different
scenarios with
hints on the choice of method.


\subsection{Notation}

The $k$ studies give rise to $p$--values,
$p_i,\;i = 1, \dots, k$.
These are assumed to be independent.
We shall also need the ordered $p$--values:
$p_{[1]} \le p_{[2]}, \dots, \le p_{[k]}$
and weights
$w_i,\;i = 1, \dots, k$.
Logarithms are natural.
A function for combining $p$--values is denoted $g$.
The size of the test is $\alpha$.
We may also need $k$ degrees of freedom, $\nu_i$.

The methods are referred to by the name of the
function in \func{metap}.
Table \ref{funcs} shows other
descriptions of each method.

\begin{table}[htbp]
\begin{tabular}{lll}
Function name & \multicolumn{2}{c}{Description(s)} \\[1ex]
 & \multicolumn{1}{c}{Eponym} \\
\func{invchisq} & Lancaster's method & Inverse chi square \\
\func{invt} & & Inverse t \\
\func{logitp} & & Logistic\\
\func{meanp} \\
\func{meanz} \\
\func{maximump} \\
\func{minimump} & Tippett's method \\
\func{sumlog} & Fisher's method & Chi square (2 df)\\
\func{sump} & Edgington's method & Uniform\\
\func{sumz} & Stouffer's method & Normal\\
\func{truncated} & Truncated Fisher\\
\func{truncated} & & rank--truncated\\
\func{votep} \\
\func{wilkinsonp} & Wilkinson's method \\
\end{tabular}
\caption{Methods considered in this document}
\label{funcs}
\end{table}

\section{Theoretical results}

There have been various attempts to clarify the
problem and to discuss optimality
of the methods.
A detailed account was provided by
\citet{liptak58}.

\citet{birnbaum54} considered the property of
admissibility.
A method is admissible if when it rejects $H_0$ for
a set of $p_i$ it will also reject $H_0$ for
$P^*_i$ where $p^*_i \le p_i$ for all $i$.
He considered that Fisher's and Tippett's method were admissible.
See also \citet{owen09}.

He also points out the problem
is poorly specified.
This may
account for the number of methods available
and their differing behaviour.
The null hypothesis $H_0$ is well defined,
that all $p_i$ have a uniform distribution on the unit interval.
There are
two classes of alternative hypothesis
\begin{itemize}
\item
$H_A$: all $p_i$ have the same (unknown)
non--uniform, non--increasing density,
\item
$H_B$:
at least one $p_i$ has an (unknown)
non--uniform, non--increasing density.
\end{itemize}

If all the tests being combined come from
what are basically replicates then $H_A$ is appropriate
whereas if they are of different kinds
of test or different conditions
then $H_B$ is appropriate.
Note that Birnbaum specifically considers the
possibility that the tests being combined may be
very different 
for instance some tests of means, some of variances,
and so on.


\section{The methods}

\subsection{Comparison scenarios}

To provide a standard of comparison
we shall use the following two situations.
Some authors have also used the case of exactly two
$p_i$.

\begin{description}
%\subsubsection{What if all $p_i = p$?\label{twopisection}}
\item[What if all $p_i = p$?]\label{twopisection}
Perhaps surprisingly there are substantial differences here
as we shall see when we look at each method.
We shall describe how the returned value
varies with $p$ and $k$.
%\subsubsection{Cancellation}
\item[Cancellation]
When the collection of primary studies
contains a number of values significant in both directions
the methods can give very different results.
If the intention of the synthesis is to examine a directional
hypothesis one would want a method where these cancelled out.
The decision between methods should be made on theoretical
grounds of course.
We shall use the following four values as our
example.
\end{description}

{\codefont
<<>>=
cancel <- c(0.001, 0.001, 0.999, 0.999)
@
}

<<echo = FALSE>>=
library(metap)
data(validity)
genvec <- function(pvals, kvals, fun, name) {
   ps <- length(pvals)
   ks <- length(kvals)
   temp <- matrix(-1, nrow = ps, ncol = ks)
   for(i in 1:ps)
   for(j in 1:ks) {
      temp[i, j] <- fun(rep(pvals[i], kvals[j]))$p
   }
   temp2 <- as.vector(temp)
   res <- data.frame(method = rep(name, length(temp2)),
      p = rep(pvals, ks),
      k = rep(kvals, each = ps),
      g = temp2
   )
   res
}
@


\subsection{Methods using transformation of the $p$--values}

One class of methods relies on transforming the $p$--values
first.

\begin{table}[htbp]
\begin{tabular}{lll}
Function name & Definition & Critical value \\[1ex]
\func{invchisq} & $\sum_{i=1}^k \chi^2_{\nu_i}(p_i)$ & $\chi^2_{\sum{\nu_i}}(\alpha)$ \\[1ex]
\func{invt} & $\frac{\sum_{i=1}^k t_{\nu_i}(p_i)}%
{\sqrt{\sum_{i=1}^k \frac{\nu_i}{\nu_i - 2}}}$ & $z(\alpha)$ \\[1ex]
\func{logitp} & $\frac{\sum_{i=1}^k \log\frac{p}{1 - p}}{C}$ & $t_{5k+4}$ \\
 & where $C = \sqrt\frac{k \pi^2 (5 k + 2)}{3(5 k + 4)}$ & \\[1ex]
\func{meanz} & $\frac{\bar{z}}{s_{\bar{z}}}$ & $t_{k-1}(\alpha)$ \\
 & where $\bar{z} = \sum_{i=1}^k \frac{z(p_i)}{k}$ \\
 & and $s_{\bar{z}} = \frac{s_z}{\sqrt{k}}$ & \\[1ex]
\func{sumlog} & $\sum_{i=1}^{k} - 2 \log p_i$ & $\chi_{2k}(\alpha)$ \\[1ex]
\func{sumz} & $\frac{\sum_{i=1}^k z(p_i)}{\sqrt{k}}$ & $z(\alpha)$\\
\end{tabular}
\caption{Definitions of methods using transformation of the $p$ values}
\label{transdefs}
\end{table}

<<echo = FALSE>>=
   kvals <- c(4, 5, 6, 8, 10, 15, 20)
   pvals <- c(0.2, 0.3, 0.3679, 0.4, 0.5, 0.6)
   dat <- rbind(
      genvec(pvals, kvals, logitp, "logitp"),
      genvec(pvals, kvals, meanz, "meanz"),
      genvec(pvals, kvals, sumlog, "sumlog"),
      genvec(pvals, kvals, sumz, "sumz")
   )
@

<<fig=TRUE,label=transeqp,include=FALSE,echo=FALSE>>=
   lattice::xyplot(g ~ k | method, groups = p, type = "l", data = dat,
      auto.key = list(space = "left", lines = TRUE, title = "p"),
      ylab = "g(p)"
   )
@

\subsubsection{The method of summation of logs, Fisher's method}

See Table \ref{transdefs} for the definition.
This works because $- 2 \log p_i$ is a $\chi^2_2$
and the sum of $\chi^2$ is itself a $\chi^2$
with degrees of freedom equal to the sum of the degrees
of freedom of the
individual $\chi^2$.
Of course the sum of the log of the $p_i$
is also the log of the product of the $p_i$.
Fisher's method \citep{fisher25} is provided in \func{sumlog}.

<<echo = FALSE>>=
set.seed(18122019)
temp <- matrix(runif(10000), nrow = 100)
fisher <- apply(temp, 1, function(x) sumlog(x)$p)
lanc4 <- apply(temp, 1, function(x) invchisq(x, 4)$p)
lanc16 <- apply(temp, 1, function(x) invchisq(x, 16)$p)
lanc256 <- apply(temp, 1, function(x) invchisq(x, 256)$p)
banda <- function(x, y) {
   res <- data.frame(sum = x + y, diff = (x - y))
   res
}
dat <- data.frame(rbind(banda(fisher, lanc4),
   banda(fisher, lanc16),
   banda(fisher, lanc256),
   banda(lanc4, lanc16),
   banda(lanc4, lanc256),
   banda(lanc16, lanc256)
   ),
   name = factor(c(rep("FL4", 100), rep("FL16", 100),
      rep("FL256", 100), rep("L4L16", 100),
      rep("L4L256", 100), rep("L16L256", 100)),
      levels = c("FL4", "FL16", "FL256", "L4L16", "L4L256", "L16L256")
   )
)
@

<<fig=TRUE,label=fishlanc,include=FALSE,echo=FALSE>>=
   lattice::xyplot(diff ~ sum | name, data = dat,
   panel = function(x, y, ...) {
      lattice::panel.xyplot(x, y, ...)
      lattice::panel.abline(h = mean(y), lty = 2)
      lattice::panel.abline(h = mean(y) + 1.96 * sd(y), lty = 3)
      lattice::panel.abline(h = mean(y) - 1.96 * sd(y), lty = 3)
   }
   )
@

<<echo = FALSE>>=
stouff <- apply(temp, 1, function(x) sumz(x)$p)
invt4 <- apply(temp, 1, function(x) invt(x, 4)$p)
invt16 <- apply(temp, 1, function(x) invt(x, 16)$p)
invt256 <- apply(temp, 1, function(x) invt(x, 256)$p)
banda <- function(x, y) {
   res <- data.frame(sum = x + y, diff = (x - y))
   res
}
dat <- data.frame(rbind(banda(stouff, invt4),
   banda(stouff, invt16),
   banda(stouff, invt256),
   banda(invt4, invt16),
   banda(invt4, invt256),
   banda(invt16, invt256)
   ),
   name = factor(c(rep("St4", 100), rep("St16", 100),
      rep("St256", 100), rep("t4t16", 100),
      rep("t4t256", 100), rep("t16t256", 100)),
      levels = c("St4", "St16", "St256", "t4t16", "t4t256", "t16t256")
   )
)
@

<<fig=TRUE,label=stouffinvt,include=FALSE,echo=FALSE>>=
lattice::xyplot(diff ~ sum | name, data = dat,
   panel = function(x, y, ...) {
      lattice::panel.xyplot(x, y, ...)
      lattice::panel.abline(h = mean(y), lty = 2)
      lattice::panel.abline(h = mean(y) + 1.96 * sd(y), lty = 3)
      lattice::panel.abline(h = mean(y) - 1.96 * sd(y), lty = 3)
   }
)
@


As can be seen in Figure \ref{equalp}
when all the $p_i=p$
\func{sumlog} returns
a value which
decreases with $k$
when $p<0.32$, increases with $k$
when $p>0.37$, and in between
increases with $k$ and then
decreases.
Some detailed algebra provided in a post
to https://stats.stackexchange.com/questions/243003 by Christoph Hanck
suggests that the breakpoint is $e^{-1} = 0.3679$.
Where the $p_i$ are less than this then for a sufficiently
large $k$ (several
hundred) the result will be significant and not if above that.
Over the range of $k$ we are plotting this bound is not
yet closely approached.

\mygraph{transeqp}{Behaviour of the methods using transformed $p$ values for $k$ values of $p=p_i$}{equalp}

\subsubsection{Inverse $\chi^2$ Lancaster's method}

It would of course be possible to generalise
Fisher's method to use
transformation to $\chi^2$ with any other number
of degrees of freedom rather than 2.
\citet{lancaster61} suggests that this is highly correlated
with \func{sumlog}.
Lancaster's method is provided in \func{invchisq}.
In fact the resemblance to \func{sumlog} becomes
less as the number of degrees of freedom increases.
Figure \ref{fishlanc} shows for a small number of selected
degrees of freedom how it compares to Fisher's method.

\twograph{fishlanc}{Fisher's method and Lancaster's method}{fishlanc}{stouffinvt}{Stouffer's method and inverse $t$}{stouffinvt}{Sum and difference plots of Fisher v Lancaster and Stouffer v inverse $t$}{compfishstouff}

\subsubsection{The method of summation of $z$ values, Stouffer's method}

The method of summation of $z$
values is provided in \func{sumz} \citep{stouffer49}.
See Table \ref{transdefs} for the definition.
As can be seen in Figure \ref{equalp}
it returns a value for our $p_i=p$
example which
decreases with $k$ when $p$ below 0.5
and increases above.

A weighted version of Stouffer's method is available
%\begin{equation}
$\frac{\sum_{i=1}^k w_i z(p_i)}{\sqrt {\sum_{i=1}^k w_i ^ 2}}$
%\end{equation}
where $w_i$ are the weights.
In the absence of effect sizes (in which case a method
using effect sizes would be more appropriate anyway)
best results are believed to be obtained with weights
proportional to the square root of the sample sizes
\citep{zaykin11} following \citet{liptak58}.


\subsubsection{Mean of normals method}

There is also a method closely related to Stouffer's using
the mean of normals provided in \func{meanz}
also defined in Table \ref{transdefs}
which has very similar properties except that
when all the $p_i$ are equal it either gives 0 or 1
as can be seen in Figure \ref{equalp}.

<<>>=
meanz(c(0.3, 0.31))$p
meanz(c(0.1, 0.2))$p
@

The method of \func{meanz} also
has the unusual property
that a set of $p$--values which are all less than those in another
set can still give rise to a larger overall $p$.
See example above.
This is the only method considered here which has this
property so if it is a desirable one then that is the only method to consider.

\subsubsection{The inverse $t$ method}

A closely related method is the inverse $t$ method.
See Table \ref{transdefs} for the definition.
This method is provided in \func{invt}.
As is clear from the definition this method tends to
Stouffer's method as $\nu_i \to \infty$.
Figure \ref{stouffinvt} shows this for selected degrees of
freedom.

\subsubsection{The method of summation of logits}

See Table \ref{transdefs} for the definition.
This method is provided in \func{logitp}.
The constant $C$ was arrived at by equating
skewness and kurtosis with that of the $t$--distribution
\citep{loughin04}.
As can be seen in Figure \ref{equalp}
this method returns a value for our $p_i=p$
example which
decreases with $k$ when $p$ below 0.5
and increases above.

\subsubsection{Examples for methods using transformations of the $p$ values}

\begin{table}[htbp]
\begin{tabular}{lll}
Function name & validity  & cancel \\[1ex]
\func{logitp} & \Sexpr{logitp(validity)$p} & \Sexpr{logitp(cancel)$p} \\
\func{meanz} & \Sexpr{meanz(validity)$p} & \Sexpr{meanz(cancel)$p} \\
\func{sumlog} & \Sexpr{sumlog(validity)$p} & \Sexpr{sumlog(cancel)$p} \\
\func{sumz} & \Sexpr{sumz(validity)$p} & \Sexpr{sumz(cancel)$p}\\
\end{tabular}
\caption{Examples of methods using transformation of the $p$ values}
\label{transexamples}
\end{table}

Using the same example dataset which we have already plotted and our cancellation
dataset we have the values in Table \ref{transexamples}.
As can be seen all the methods cancel except for \func{sumlog}.
The agreement for the validity dataset is close.
Lancaster's method and inverse $t$ are not shown as they are both
infinite families of possible methods
and in any event are similar to Fisher's method
and Stouffer's method respectively.

\subsection{Methods using untransformed $p$--values}

\begin{table}[htbp]
\begin{tabular}{lll}
Function name & Definition & Critical value \\[1ex]
\func{meanp} & $\bar p = \frac{\sum_{i=1}^k p_i}{k}$ \\
 & $z = (0.5 - \bar{p}) \sqrt{12k}$ & $z(\alpha)$ \\
\func{minimump} & $p_{[1]}$ & $1 - (1 - \alpha)^{\frac{1}{k}}$ \\
\func{maximump} & $p_{[k]}$ & $\alpha^k$ \\
\func{wilkinsonp} & $p_{[r]}$ & $\sum_{s=r}^k {k \choose s}\alpha^s (1 - \alpha)^{k-s}$\\[1ex]
\func{sump} & $\frac{(S)^k}{k!}%
- {k - 1 \choose 1}\frac{(S - 1)^k}{k!}%
+ {k - 2 \choose 2}\frac{(S - 2)^k}{k!} - \dots$ & $\alpha$ \\
 & where $S = \sum_{i=1}^k p_i$ \\
\end{tabular}
\caption{Definitions of methods not using transformation of the $p$ values, %
the series for \func{sump} continues
until the term in in the numerator $(S-i)$
becomes negative}
\label{untransdefs}
\end{table}


<<echo = FALSE>>=
   kvals <- c(4, 5, 6, 8, 10, 15, 20)
   pvals <- c(0.2, 0.3, 0.3679, 0.4, 0.5, 0.6)
   dat <- rbind(
      genvec(pvals, kvals, meanp, "meanp"),
      genvec(pvals, kvals, maximump, "maximump"),
      genvec(pvals, kvals, minimump, "minimump"),
      genvec(pvals, kvals, sump, "sump"),
      genvec(pvals, kvals, votep, "votep")
   )
@

<<fig=TRUE,label=untranseqp,include=FALSE,echo=FALSE>>=
   lattice::xyplot(g ~ k | method, groups = p, type = "l", data = dat,
      auto.key = list(space = "left", lines = TRUE, title = "p"),
      ylab = "g(p)"
   )
@

\mygraph{untranseqp}{Behaviour of the methods using untransformed $p$ values for $k$ values of $p=p_i$}{unequalp}


\subsubsection{The method of minimum $p$, maximum $p$, and Wilkinson's method}

The methods of minimum $p$ \citep{tippett31}, maximum $p$ and Wilkinson \citep{wilkinson51}
are defined in Table \ref{untransdefs}.
Wilkinson's method depends on which value
(the $r$th) of $p_{[i]}$ is
selected.
% p is pbeta(p[r], r, k+1-r)
% critical p is qbeta(alpha, r, k+1-r)
Wilkinson's method is provided in \func{wilkinsonp} and a
convenience function \func{minimump}
with its own \code{print} method is provided for
the minimum $p$ method ($r=1$).
It is also possible to use the method for the
maximum $p$ (that is $r=k$) and a convenience function \func{maximump}
is provided for that purpose.

As can be seen in Figure \ref{unequalp}
these methods return a value for our $p_i=p$
example which
always increases with $k$
which is true for
\func{minimump}
and
which always decreases with $k$
which is true for
\func{maximump}

\subsubsection{The method of summation of $p$--values, Edgington's method\label{sump}}

Defined in Table \ref{untransdefs}
\citep{edgington72a}.
This method is provided in \func{sump}.
As can be seen in Figure \ref{unequalp}
this method returns a value for our $p_i=p$
example which
decreases with $k$ when $p$ below 0.5
and increases above.

Some authors use a simpler version, $\frac{(\sum p)^k}{k!}$,
for instance \citet{rosenthal78} in the text
although compare his Table 4.
This can be very conservative when
$\sum p > 1$
There seems no particular need to use this method but
it is returned by \func{sump}
as the value of \code{conservativep}
for use in checking published values.

Note also that there can be numerical problems for extreme values
of $S$ and in that case recourse might be made to
\func{meanp} which has similar
properties.

\subsubsection{The mean $p$ method}

Defined in Table \ref{untransdefs}.
Although this method is attributed to Edgington \citep{edgington72b}
when the phrase Edgington's method is used
it refers to the method of summation of $p$--values
described above in Section \ref{sump}.
As can be seen in Figure \ref{unequalp}
this method returns a value for our $p_i=p$
example which
decreases with $k$ when $p$ below 0.5
and increases above.

Not surprisingly this method gives very similar results
to Edington's other method implemented
in \func{sump} and since it does not have the
numerical problems of that method it might perhaps be preferred.

\subsubsection{Examples for methods using untransformed $p$--values}

Using the same example dataset which we have already plotted and our cancellation
dataset we have the values in Table \ref{untransexamples}.
As can be seen \func{meanp} and \func{sump} cancel but the
other two do not.
Agreement here is not so good especially for the maximum p method.
Wilkinson's method not shown as it depends on the value of $r$.

\begin{table}[htbp]
\begin{tabular}{lll}
Function name & validity  & cancel \\[1ex]
\func{minimump} & \Sexpr{minimump(validity)$p} & \Sexpr{minimump(cancel)$p} \\
\func{maximump} & \Sexpr{maximump(validity)$p} & \Sexpr{maximump(cancel)$p} \\
\func{meanp} & \Sexpr{meanp(validity)$p} & \Sexpr{meanp(cancel)$p}\\
\func{sump} & \Sexpr{sump(validity)$p} & \Sexpr{sump(cancel)$p} \\
\end{tabular}
\caption{Examples for methods using the untransformed $p$ values}
\label{untransexamples}
\end{table}

\subsection{Other methods}

\subsubsection{The method of vote--counting}

A simple way of looking at the problem is
vote counting.
Strictly speaking this is not a method which combines
$p$--values in the same sense as the other methods.
If most of the studies have produced results in favour of the
alternative hypothesis irrespective of whether any of them is
individually significant then that might be regarded as evidence
for that alternative.
The numbers for and against may be compared with what
would be expected under the null using the binomial distribution.
A variation on this would allow for a neutral zone of studies
which are considered neither for nor against.
For instance one might only
count studies which have reached some conventional level of
statistical significance in the two different directions.

This method returns a value for our $p_i=p$
example which
is 1 for $p$
values above 0.5 and otherwise invariant with $p$
but decreases with $k$.
This method does cancel significant values in both
directions.

\begin{table}[htbp]
\begin{tabular}{lll}
Function name & validity  & cancel \\[1ex]
\func{votep} & \Sexpr{votep(validity)$p} & \Sexpr{votep(cancel)$p} \\
\end{tabular}
\caption{Examples for vote counting}
\label{votepexamples}
\end{table}

\subsubsection{Methods not using all $p$--values}

If there is a hypothesis that the signal will be
concentrated in only a few $p$--values then alternative methods
are available in \func{truncated}.
This is a wrapper to two packages available on
CRAN: \pkg{TFisher} which provides the
truncated Fisher method \citep{zaykin07,zhang18}
and \pkg{mutoss} which provides
the rank--truncated Fisher
method \citep{dudbridge03}.
Note that Table \ref{truncatedexamples} only shows
results for the validity data--set as, since the
methods explicitly only consider results in one direction
the cancellation issue does not arise.

\begin{table}[htbp]
\begin{tabular}{lll}
Function name & truncated at $p$ = 0.5  & truncated at rank = 5 \\[1ex]
\func{truncated} & \Sexpr{truncated(validity, ptrunc = 0.5)$p} & \Sexpr{truncated(validity, rtrunc = 5)$p} \\
\end{tabular}
\caption{Examples for truncated using the validity data--set}
\label{truncatedexamples}
\end{table}

These methods are appropriate for the situation where it is
known that many of the $p$--values are noise and
there will only be a few signals.

\section{Loughin's recommendations}

In his simulation study \citet{loughin04} carried out extensive comparisons.
Note that he did not consider all the methods implemented here.
These omissions are not too important for our purposes.
The methods implemented here as \func{invchisq}, \func{invt},
\func{meanp} and \func{meanz} are all very similar to ones which he did
study.
The truncation methods appeared about the same time as
his work but in any case are fundamentally different.
Vote counting is arguably not a method of the same sort.

As Loghin points out
the first thing to consider is whether large $p$--values
should cancel small ones.
If this is not desired then the only methods to
consider are those in \func{sumlog} (Fisher),
\func{minimump} (Tippett) and \func{maximump}.

\mypdf{struct}{Loughin's recommendations based on structure}{struct}{[height=6cm,width=8cm]}

He bases his recommendations on criteria of structure and the
arrangement of evidence against $H_0$.
Figure \ref{struct} shows a summary of his recommendations
about the structure of the evidence.

\mypdf{strength}{Loughin's recommendations based on where the strength of the evidence is located}{strength}{[height=8cm,width=12cm]}

Figure \ref{strength} summarise his recommendations
about the arrangement of evidence.

Overall he considered the choice to lie between
Stouffer's method, Fisher's method
and the logistic method
implemented in \func{logitp}.
As has already been mentioned Fisher's method cancels
whereas the other two do not so if the weak evidence
in a small number of $p$--values is not to be
over--whelmed by the others then Fisher is the
best choice.
However where the evidence is more evenly spread
Stouffer's method may be preferred.
The logistic method represents a compromise between
them and is perhaps best suited where the pattern
of evidence is not clear in advance.
The other methods are not universally ruled out
and may be helpful in the specific circumstance
outlined in his summaries.


\bibliography{metap}
\bibliographystyle{plainnat}
\end{document}

\documentclass[12pt]{article}
\usepackage{amsmath,amssymb}
%\usepackage{mydef2}
\usepackage[round]{natbib}
\usepackage{parskip,url}
\usepackage{graphicx,subfig}
\setlength{\topmargin}{0cm}
\addtolength{\textheight}{2cm}
%\lhead{}
%\input{title}
%\VignetteIndexEntry{Comparison of methods in the metap package}
\title{Comparison of methods in the \pkg{metap} package}
\author{Michael Dewey}
\newcommand{\pkg}[1]{\texttt{#1}}
\newcommand{\func}[1]{\texttt{#1}}
\newcommand{\code}[1]{\texttt{#1}}
\newcommand{\codefont}{\footnotesize}
\newcommand{\mygraph}[3]{%
\begin{figure}[htbp]
\includegraphics[height=6cm,width=10cm]{compare-#1}
\caption{#2}
\label{#3}
\end{figure}
}
% in mypdf fourth parameter needs []
\newcommand{\mypdf}[4]{%
\begin{figure}[htbp]
\includegraphics#4{#1.pdf}
\caption{#2}
\label{#3}
\end{figure}
}
\newcommand{\twograph}[8]{%
\begin{figure}[htbp]
\subfloat[#2\label{#3}]{\includegraphics[height=6cm,width=7cm]{compare-#1}}%
\subfloat[#5\label{#6}]{\includegraphics[height=6cm,width=7cm]{compare-#4}}
\caption{#7}
\label{#8}
\end{figure}
}
\begin{document}
\maketitle

\section{Introduction}

\subsection{What is this document for?}

This document describes some methods for the meta--analysis of
$p$--values
(significance values)
contained in the package \pkg{metap}
and contains comments on
the performance of the various algorithms
under a small number of different
scenarios with
hints on the choice of method.


\subsection{Notation}

The $k$ studies give rise to $p$--values,
$p_i,\;i = 1, \dots, k$.
These are assumed to be independent.
We shall also need the ordered $p$--values:
$p_{[1]} \le p_{[2]}, \dots, \le p_{[k]}$
and weights
$w_i,\;i = 1, \dots, k$.
Logarithms are natural.
A function for combining $p$--values is denoted $g$.
The size of the test is $\alpha$.
We may also need $k$ degrees of freedom, $\nu_i$.

The methods are referred to by the name of the
function in \func{metap}.
Table \ref{funcs} shows other
descriptions of each method.

\begin{table}[htbp]
\begin{tabular}{lll}
Function name & \multicolumn{2}{c}{Description(s)} \\[1ex]
 & \multicolumn{1}{c}{Eponym} \\
\func{invchisq} & Lancaster's method & Inverse chi square \\
\func{invt} & & Inverse t \\
\func{logitp} & & Logistic\\
\func{meanp} \\
\func{meanz} \\
\func{maximump} \\
\func{minimump} & Tippett's method \\
\func{sumlog} & Fisher's method & Chi square (2 df)\\
\func{sump} & Edgington's method & Uniform\\
\func{sumz} & Stouffer's method & Normal\\
\func{truncated} & Truncated Fisher\\
\func{truncated} & & rank--truncated\\
\func{votep} \\
\func{wilkinsonp} & Wilkinson's method \\
\end{tabular}
\caption{Methods considered in this document}
\label{funcs}
\end{table}

\section{Theoretical results}

There have been various attempts to clarify the
problem and to discuss optimality
of the methods.
A detailed account was provided by
\citet{liptak58}.

\citet{birnbaum54} considered the property of
admissibility.
A method is admissible if when it rejects $H_0$ for
a set of $p_i$ it will also reject $H_0$ for
$P^*_i$ where $p^*_i \le p_i$ for all $i$.
He considered that Fisher's and Tippett's method were admissible.
See also \citet{owen09}.

He also points out the problem
is poorly specified.
This may
account for the number of methods available
and their differing behaviour.
The null hypothesis $H_0$ is well defined,
that all $p_i$ have a uniform distribution on the unit interval.
There are
two classes of alternative hypothesis
\begin{itemize}
\item
$H_A$: all $p_i$ have the same (unknown)
non--uniform, non--increasing density,
\item
$H_B$:
at least one $p_i$ has an (unknown)
non--uniform, non--increasing density.
\end{itemize}

If all the tests being combined come from
what are basically replicates then $H_A$ is appropriate
whereas if they are of different kinds
of test or different conditions
then $H_B$ is appropriate.
Note that Birnbaum specifically considers the
possibility that the tests being combined may be
very different 
for instance some tests of means, some of variances,
and so on.


\section{The methods}

\subsection{Comparison scenarios}

To provide a standard of comparison
we shall use the following two situations.
Some authors have also used the case of exactly two
$p_i$.

\begin{description}
%\subsubsection{What if all $p_i = p$?\label{twopisection}}
\item[What if all $p_i = p$?]\label{twopisection}
Perhaps surprisingly there are substantial differences here
as we shall see when we look at each method.
We shall describe how the returned value
varies with $p$ and $k$.
%\subsubsection{Cancellation}
\item[Cancellation]
When the collection of primary studies
contains a number of values significant in both directions
the methods can give very different results.
If the intention of the synthesis is to examine a directional
hypothesis one would want a method where these cancelled out.
The decision between methods should be made on theoretical
grounds of course.
We shall use the following four values as our
example.
\end{description}

{\codefont
<<>>=
cancel <- c(0.001, 0.001, 0.999, 0.999)
@
}

<<echo = FALSE>>=
library(metap)
data(dat.metap)
validity <- dat.metap$validity$p
genvec <- function(pvals, kvals, fun, name) {
   ps <- length(pvals)
   ks <- length(kvals)
   temp <- matrix(-1, nrow = ps, ncol = ks)
   for(i in 1:ps)
   for(j in 1:ks) {
      temp[i, j] <- fun(rep(pvals[i], kvals[j]))$p
   }
   temp2 <- as.vector(temp)
   res <- data.frame(method = rep(name, length(temp2)),
      p = rep(pvals, ks),
      k = rep(kvals, each = ps),
      g = temp2
   )
   res
}
@


\subsection{Methods using transformation of the $p$--values}

One class of methods relies on transforming the $p$--values
first.

\begin{table}[htbp]
\begin{tabular}{lll}
Function name & Definition & Critical value \\[1ex]
\func{invchisq} & $\sum_{i=1}^k \chi^2_{\nu_i}(p_i)$ & $\chi^2_{\sum{\nu_i}}(\alpha)$ \\[1ex]
\func{invt} & $\frac{\sum_{i=1}^k t_{\nu_i}(p_i)}%
{\sqrt{\sum_{i=1}^k \frac{\nu_i}{\nu_i - 2}}}$ & $z(\alpha)$ \\[1ex]
\func{logitp} & $\frac{\sum_{i=1}^k \log\frac{p}{1 - p}}{C}$ & $t_{5k+4}$ \\
 & where $C = \sqrt\frac{k \pi^2 (5 k + 2)}{3(5 k + 4)}$ & \\[1ex]
\func{meanz} & $\frac{\bar{z}}{s_{\bar{z}}}$ & $t_{k-1}(\alpha)$ \\
 & where $\bar{z} = \sum_{i=1}^k \frac{z(p_i)}{k}$ \\
 & and $s_{\bar{z}} = \frac{s_z}{\sqrt{k}}$ & \\[1ex]
\func{sumlog} & $\sum_{i=1}^{k} - 2 \log p_i$ & $\chi_{2k}(\alpha)$ \\[1ex]
\func{sumz} & $\frac{\sum_{i=1}^k z(p_i)}{\sqrt{k}}$ & $z(\alpha)$\\
\end{tabular}
\caption{Definitions of methods using transformation of the $p$ values}
\label{transdefs}
\end{table}

<<echo = FALSE>>=
   kvals <- c(4, 5, 6, 8, 10, 15, 20)
   pvals <- c(0.2, 0.3, 0.3679, 0.4, 0.5, 0.6)
   dat <- rbind(
      genvec(pvals, kvals, logitp, "logitp"),
      genvec(pvals, kvals, meanz, "meanz"),
      genvec(pvals, kvals, sumlog, "sumlog"),
      genvec(pvals, kvals, sumz, "sumz")
   )
@

<<fig=TRUE,label=transeqp,include=FALSE,echo=FALSE>>=
   lattice::xyplot(g ~ k | method, groups = p, type = "l", data = dat,
      auto.key = list(space = "left", lines = TRUE, title = "p"),
      ylab = "g(p)"
   )
@

\subsubsection{The method of summation of logs, Fisher's method}

See Table \ref{transdefs} for the definition.
This works because $- 2 \log p_i$ is a $\chi^2_2$
and the sum of $\chi^2$ is itself a $\chi^2$
with degrees of freedom equal to the sum of the degrees
of freedom of the
individual $\chi^2$.
Of course the sum of the log of the $p_i$
is also the log of the product of the $p_i$.
Fisher's method \citep{fisher25} is provided in \func{sumlog}.

<<echo = FALSE>>=
set.seed(18122019)
temp <- matrix(runif(10000), nrow = 100)
fisher <- apply(temp, 1, function(x) sumlog(x)$p)
lanc4 <- apply(temp, 1, function(x) invchisq(x, 4)$p)
lanc16 <- apply(temp, 1, function(x) invchisq(x, 16)$p)
lanc256 <- apply(temp, 1, function(x) invchisq(x, 256)$p)
banda <- function(x, y) {
   res <- data.frame(sum = x + y, diff = (x - y))
   res
}
dat <- data.frame(rbind(banda(fisher, lanc4),
   banda(fisher, lanc16),
   banda(fisher, lanc256),
   banda(lanc4, lanc16),
   banda(lanc4, lanc256),
   banda(lanc16, lanc256)
   ),
   name = factor(c(rep("FL4", 100), rep("FL16", 100),
      rep("FL256", 100), rep("L4L16", 100),
      rep("L4L256", 100), rep("L16L256", 100)),
      levels = c("FL4", "FL16", "FL256", "L4L16", "L4L256", "L16L256")
   )
)
@

<<fig=TRUE,label=fishlanc,include=FALSE,echo=FALSE>>=
   lattice::xyplot(diff ~ sum | name, data = dat,
   panel = function(x, y, ...) {
      lattice::panel.xyplot(x, y, ...)
      lattice::panel.abline(h = mean(y), lty = 2)
      lattice::panel.abline(h = mean(y) + 1.96 * sd(y), lty = 3)
      lattice::panel.abline(h = mean(y) - 1.96 * sd(y), lty = 3)
   }
   )
@

<<echo = FALSE>>=
stouff <- apply(temp, 1, function(x) sumz(x)$p)
invt4 <- apply(temp, 1, function(x) invt(x, 4)$p)
invt16 <- apply(temp, 1, function(x) invt(x, 16)$p)
invt256 <- apply(temp, 1, function(x) invt(x, 256)$p)
banda <- function(x, y) {
   res <- data.frame(sum = x + y, diff = (x - y))
   res
}
dat <- data.frame(rbind(banda(stouff, invt4),
   banda(stouff, invt16),
   banda(stouff, invt256),
   banda(invt4, invt16),
   banda(invt4, invt256),
   banda(invt16, invt256)
   ),
   name = factor(c(rep("St4", 100), rep("St16", 100),
      rep("St256", 100), rep("t4t16", 100),
      rep("t4t256", 100), rep("t16t256", 100)),
      levels = c("St4", "St16", "St256", "t4t16", "t4t256", "t16t256")
   )
)
@

<<fig=TRUE,label=stouffinvt,include=FALSE,echo=FALSE>>=
lattice::xyplot(diff ~ sum | name, data = dat,
   panel = function(x, y, ...) {
      lattice::panel.xyplot(x, y, ...)
      lattice::panel.abline(h = mean(y), lty = 2)
      lattice::panel.abline(h = mean(y) + 1.96 * sd(y), lty = 3)
      lattice::panel.abline(h = mean(y) - 1.96 * sd(y), lty = 3)
   }
)
@


As can be seen in Figure \ref{equalp}
when all the $p_i=p$
\func{sumlog} returns
a value which
decreases with $k$
when $p<0.32$, increases with $k$
when $p>0.37$, and in between
increases with $k$ and then
decreases.
Some detailed algebra provided in a post
to https://stats.stackexchange.com/questions/243003 by Christoph Hanck
suggests that the breakpoint is $e^{-1} = 0.3679$.
Where the $p_i$ are less than this then for a sufficiently
large $k$ (several
hundred) the result will be significant and not if above that.
Over the range of $k$ we are plotting this bound is not
yet closely approached.

\mygraph{transeqp}{Behaviour of the methods using transformed $p$ values for $k$ values of $p=p_i$}{equalp}

\subsubsection{Inverse $\chi^2$ Lancaster's method}

It would of course be possible to generalise
Fisher's method to use
transformation to $\chi^2$ with any other number
of degrees of freedom rather than 2.
\citet{lancaster61} suggests that this is highly correlated
with \func{sumlog}.
Lancaster's method is provided in \func{invchisq}.
In fact the resemblance to \func{sumlog} becomes
less as the number of degrees of freedom increases.
Figure \ref{fishlanc} shows for a small number of selected
degrees of freedom how it compares to Fisher's method.

\twograph{fishlanc}{Fisher's method and Lancaster's method}{fishlanc}{stouffinvt}{Stouffer's method and inverse $t$}{stouffinvt}{Sum and difference plots of Fisher v Lancaster and Stouffer v inverse $t$}{compfishstouff}

\subsubsection{The method of summation of $z$ values, Stouffer's method}

The method of summation of $z$
values is provided in \func{sumz} \citep{stouffer49}.
See Table \ref{transdefs} for the definition.
As can be seen in Figure \ref{equalp}
it returns a value for our $p_i=p$
example which
decreases with $k$ when $p$ below 0.5
and increases above.

A weighted version of Stouffer's method is available
%\begin{equation}
$\frac{\sum_{i=1}^k w_i z(p_i)}{\sqrt {\sum_{i=1}^k w_i ^ 2}}$
%\end{equation}
where $w_i$ are the weights.
In the absence of effect sizes (in which case a method
using effect sizes would be more appropriate anyway)
best results are believed to be obtained with weights
proportional to the square root of the sample sizes
\citep{zaykin11} following \citet{liptak58}.


\subsubsection{Mean of normals method}

There is also a method closely related to Stouffer's using
the mean of normals provided in \func{meanz}
also defined in Table \ref{transdefs}
which has very similar properties except that
when all the $p_i$ are equal it either gives 0 or 1
as can be seen in Figure \ref{equalp}.

<<>>=
meanz(c(0.3, 0.31))$p
meanz(c(0.1, 0.2))$p
@

The method of \func{meanz} also
has the unusual property
that a set of $p$--values which are all less than those in another
set can still give rise to a larger overall $p$.
See example above.
This is the only method considered here which has this
property so if it is a desirable one then that is the only method to consider.

\subsubsection{The inverse $t$ method}

A closely related method is the inverse $t$ method.
See Table \ref{transdefs} for the definition.
This method is provided in \func{invt}.
As is clear from the definition this method tends to
Stouffer's method as $\nu_i \to \infty$.
Figure \ref{stouffinvt} shows this for selected degrees of
freedom.

\subsubsection{The method of summation of logits}

See Table \ref{transdefs} for the definition.
This method is provided in \func{logitp}.
The constant $C$ was arrived at by equating
skewness and kurtosis with that of the $t$--distribution
\citep{loughin04}.
As can be seen in Figure \ref{equalp}
this method returns a value for our $p_i=p$
example which
decreases with $k$ when $p$ below 0.5
and increases above.

\subsubsection{Examples for methods using transformations of the $p$ values}

<<echo=FALSE>>=
log10p <- function(x) {
   res <- round(-log(x, base = 10), 2)
   res
}
@
\begin{table}[htbp]
\begin{tabular}{lll}
Function name & \multicolumn{1}{c}{validity}  & \multicolumn{1}{c}{cancel} \\
 & \multicolumn{1}{c}{value expressed} \\
 & \multicolumn{1}{c}{as $-\log_{10}p$}  \\[1ex]
\func{logitp} & \Sexpr{log10p(logitp(validity)$p)} & \Sexpr{logitp(cancel)$p} \\
\func{meanz} & \Sexpr{log10p(meanz(validity)$p)} & \Sexpr{meanz(cancel)$p} \\
\func{sumlog} & \Sexpr{log10p(sumlog(validity)$p)} & \Sexpr{round(sumlog(cancel)$p, 5)} \\
\func{sumz} & \Sexpr{log10p(sumz(validity)$p)} & \Sexpr{sumz(cancel)$p}\\
\end{tabular}
\caption{Examples of methods using transformation of the $p$ values}
\label{transexamples}
\end{table}

Using the same example dataset which we have already plotted and our cancellation
dataset we have the values in Table \ref{transexamples}.
As can be seen all the methods cancel except for \func{sumlog}.
The agreement for the validity dataset is close except
for \func{meanz} whoch gives a value several orders of magnitude
greater than the other three.
Lancaster's method and inverse $t$ are not shown as they are both
infinite families of possible methods
and in any event are similar to Fisher's method
and Stouffer's method respectively.

\subsection{Methods using untransformed $p$--values}

\begin{table}[htbp]
\begin{tabular}{lll}
Function name & Definition & Critical value \\[1ex]
\func{meanp} & $\bar p = \frac{\sum_{i=1}^k p_i}{k}$ \\
 & $z = (0.5 - \bar{p}) \sqrt{12k}$ & $z(\alpha)$ \\
\func{minimump} & $p_{[1]}$ & $1 - (1 - \alpha)^{\frac{1}{k}}$ \\
\func{maximump} & $p_{[k]}$ & $\alpha^k$ \\
\func{wilkinsonp} & $p_{[r]}$ & $\sum_{s=r}^k {k \choose s}\alpha^s (1 - \alpha)^{k-s}$\\[1ex]
\func{sump} & $\frac{(S)^k}{k!}%
- {k \choose 1}\frac{(S - 1)^k}{k!}%
+ {k \choose 2}\frac{(S - 2)^k}{k!} - \dots$ & $\alpha$ \\
 & where $S = \sum_{i=1}^k p_i$ \\
\end{tabular}
\caption{Definitions of methods not using transformation of the $p$ values, %
the series for \func{sump} continues
until the term in in the numerator $(S-i)$
becomes negative}
\label{untransdefs}
\end{table}


<<echo = FALSE>>=
   kvals <- c(4, 5, 6, 8, 10, 15, 20)
   pvals <- c(0.2, 0.3, 0.3679, 0.4, 0.5, 0.6)
   dat <- rbind(
      genvec(pvals, kvals, meanp, "meanp"),
      genvec(pvals, kvals, maximump, "maximump"),
      genvec(pvals, kvals, minimump, "minimump"),
      genvec(pvals, kvals, sump, "sump"),
      genvec(pvals, kvals, votep, "votep")
   )
@

<<fig=TRUE,label=untranseqp,include=FALSE,echo=FALSE>>=
   lattice::xyplot(g ~ k | method, groups = p, type = "l", data = dat,
      auto.key = list(space = "left", lines = TRUE, title = "p"),
      ylab = "g(p)"
   )
@

\mygraph{untranseqp}{Behaviour of the methods using untransformed $p$ values for $k$ values of $p=p_i$}{unequalp}


\subsubsection{The method of minimum $p$, maximum $p$, and Wilkinson's method}

The methods of minimum $p$ \citep{tippett31}, maximum $p$ and Wilkinson \citep{wilkinson51}
are defined in Table \ref{untransdefs}.
Wilkinson's method depends on which value
(the $r$th) of $p_{[i]}$ is
selected.
% p is pbeta(p[r], r, k+1-r)
% critical p is qbeta(alpha, r, k+1-r)
Wilkinson's method is provided in \func{wilkinsonp} and a
convenience function \func{minimump}
with its own \code{print} method is provided for
the minimum $p$ method ($r=1$).
It is also possible to use the method for the
maximum $p$ (that is $r=k$) and a convenience function \func{maximump}
is provided for that purpose.

As can be seen in Figure \ref{unequalp}
these methods return a value for our $p_i=p$
example which
always increases with $k$
which is true for
\func{minimump}
and
which always decreases with $k$
which is true for
\func{maximump}

\subsubsection{The method of summation of $p$--values, Edgington's method\label{sump}}

Defined in Table \ref{untransdefs}
\citep{edgington72a}.
This method is provided in \func{sump}.
As can be seen in Figure \ref{unequalp}
this method returns a value for our $p_i=p$
example which
decreases with $k$ when $p$ below 0.5
and increases above.

Some authors use a simpler version, $\frac{(\sum p)^k}{k!}$,
for instance \citet{rosenthal78} in the text
although compare his Table 4.
This can be very conservative when
$\sum p > 1$
There seems no particular need to use this method but
it is returned by \func{sump}
as the value of \code{conservativep}
for use in checking published values.

Note also that there can be numerical problems for extreme values
of $S$ and in that case recourse might be made to
\func{meanp} which has similar
properties.

\subsubsection{The mean $p$ method}

Defined in Table \ref{untransdefs}.
Although this method is attributed to Edgington \citep{edgington72b}
when the phrase Edgington's method is used
it refers to the method of summation of $p$--values
described above in Section \ref{sump}.
As can be seen in Figure \ref{unequalp}
this method returns a value for our $p_i=p$
example which
decreases with $k$ when $p$ below 0.5
and increases above.

Not surprisingly this method gives very similar results
to Edington's other method implemented
in \func{sump} and since it does not have the
numerical problems of that method it might perhaps be preferred.

\subsubsection{Examples for methods using untransformed $p$--values}

Using the same example dataset which we have already plotted and our cancellation
dataset we have the values in Table \ref{untransexamples}.
As can be seen \func{meanp} and \func{sump} cancel but the
other two do not.
Agreement here is not so good especially for the maximum p method.
Wilkinson's method not shown as it depends on the value of $r$.

\begin{table}[htbp]
\begin{tabular}{lrl}
Function name & \multicolumn{1}{c}{validity}  & \multicolumn{1}{c}{cancel} \\
 & \multicolumn{1}{c}{value expressed} \\
 & \multicolumn{1}{c}{as $-\log_{10}p$}  \\[1ex]
 \\[1ex]
\func{minimump} & \Sexpr{log10p(minimump(validity)$p)} & \Sexpr{round(minimump(cancel)$p, 5)} \\
\func{maximump} & \Sexpr{log10p(maximump(validity)$p)} & \Sexpr{round(maximump(cancel)$p, 5)} \\
\func{meanp} & \Sexpr{log10p(meanp(validity)$p)} & \Sexpr{meanp(cancel)$p}\\
\func{sump} & \Sexpr{log10p(sump(validity)$p)} & \Sexpr{sump(cancel)$p} \\
\end{tabular}
\caption{Examples for methods using the untransformed $p$ values}
\label{untransexamples}
\end{table}

\subsection{Other methods}

\subsubsection{The method of vote--counting}

A simple way of looking at the problem is
vote counting.
Strictly speaking this is not a method which combines
$p$--values in the same sense as the other methods.
If most of the studies have produced results in favour of the
alternative hypothesis irrespective of whether any of them is
individually significant then that might be regarded as evidence
for that alternative.
The numbers for and against may be compared with what
would be expected under the null using the binomial distribution.
A variation on this would allow for a neutral zone of studies
which are considered neither for nor against.
For instance one might only
count studies which have reached some conventional level of
statistical significance in the two different directions.

This method returns a value for our $p_i=p$
example which
is 1 for $p$
values above 0.5 and otherwise invariant with $p$
but decreases with $k$.
This method does cancel significant values in both
directions.

\begin{table}[htbp]
\begin{tabular}{lll}
Function name & validity  & cancel \\[1ex]
\func{votep} & \Sexpr{round(votep(validity)$p, 6)} &
 \Sexpr{round(votep(cancel)$p, 5)} \\
\end{tabular}
\caption{Examples for vote counting}
\label{votepexamples}
\end{table}

\subsubsection{Methods not using all $p$--values}

If there is a hypothesis that the signal will be
concentrated in only a few $p$--values then alternative methods
are available in \func{truncated}.
This is a wrapper to two packages available on
CRAN: \pkg{TFisher} which provides the
truncated Fisher method \citep{zaykin07,zhang18}
and \pkg{mutoss} which provides
the rank--truncated Fisher
method \citep{dudbridge03}.
Note that Table \ref{truncatedexamples} only shows
results for the validity data--set as, since the
methods explicitly only consider results in one direction
the cancellation issue does not arise.

\begin{table}[htbp]
\begin{tabular}{lll}
Function name & truncated at $p$ = 0.5  & truncated at rank = 5 \\[1ex]
\func{truncated} & \Sexpr{log10p(truncated(validity, ptrunc = 0.5)$p)}
 & \Sexpr{log10p(truncated(validity, rtrunc = 5)$p)} \\
\end{tabular}
\caption{Examples for truncated using the validity data--set
 expressed as $-\log_{10}p$}
\label{truncatedexamples}
\end{table}

\citet{dudbridge03} compare these two methods.
They comment that in a meta--analysis the method
using truncated Fisher may be preferred particularly
if reporting bias is suspected.
When the interest is in a small set of signals
in the presence of much noise as occurs in
genome wide association scanning then the rank
truncation method is recommended.
Their article provides more details about
the methods.

\section{Loughin's recommendations}

In his simulation study \citet{loughin04} carried out extensive comparisons.
Note that he did not consider all the methods implemented here.
These omissions are not too important for our purposes.
The methods implemented here as \func{invchisq}, \func{invt},
\func{meanp} and \func{meanz} are all very similar to ones which he did
study.
The truncation methods appeared about the same time as
his work but in any case are fundamentally different.
Vote counting is arguably not a method of the same sort.

As Loughin points out
the first thing to consider is whether large $p$--values
should cancel small ones.
If this is not desired then the only methods to
consider are those in \func{sumlog} (Fisher),
\func{minimump} (Tippett) and \func{maximump}.

\mypdf{struct}{Loughin's recommendations based on structure}{struct}{[height=6cm,width=8cm]}

He bases his recommendations on criteria of structure and the
arrangement of evidence against $H_0$.
Figure \ref{struct} shows a summary of his recommendations
about the structure of the evidence.

\mypdf{strength}{Loughin's recommendations based on where the strength of the evidence is located}{strength}{[height=8cm,width=12cm]}

Figure \ref{strength} summarise his recommendations
about the arrangement of evidence.

Overall he considered the choice to lie between
Stouffer's method, Fisher's method
and the logistic method
implemented in \func{logitp}.
As has already been mentioned Fisher's method cancels
whereas the other two do not so if the weak evidence
in a small number of $p$--values is not to be
over--whelmed by the others then Fisher is the
best choice.
However where the evidence is more evenly spread
Stouffer's method may be preferred.
The logistic method represents a compromise between
them and is perhaps best suited where the pattern
of evidence is not clear in advance.
The other methods are not universally ruled out
and may be helpful in the specific circumstance
outlined in his summaries.


\bibliography{metap}
\bibliographystyle{plainnat}
\end{document}

\documentclass[12pt]{article}
\usepackage{amsmath,amssymb}
%\usepackage{mydef2}
\usepackage[round]{natbib}
\usepackage{parskip,url}
\usepackage{graphicx,subfig}
\setlength{\topmargin}{0cm}
\addtolength{\textheight}{2cm}
%\lhead{}
%\input{title}
%\VignetteIndexEntry{Introduction to the metap package}
\title{Introduction to the \pkg{metap} package}
\author{Michael Dewey}
\newcommand{\pkg}[1]{\texttt{#1}}
\newcommand{\func}[1]{\texttt{#1}}
\newcommand{\code}[1]{\texttt{#1}}
\newcommand{\codefont}{\footnotesize}
\newcommand{\mygraph}[3]{%
\begin{figure}[htbp]
\includegraphics[height=6cm,width=10cm]{#1}
\caption{#2}
\label{#3}
\end{figure}
}
\newcommand{\twograph}[8]{%
\begin{figure}[htbp]
\subfloat[#2\label{#3}]{\includegraphics[height=6cm,width=7cm]{#1}}%
\subfloat[#5\label{#6}]{\includegraphics[height=6cm,width=7cm]{#4}}
\caption{#7}
\label{#8}
\end{figure}
}
\begin{document}
\maketitle

\section{Introduction}

\subsection{What is this document for?}

This document describes some methods for the meta--analysis of
$p$--values
(significance values)
and their implementation
in the package \pkg{metap}.
I welcome feedback about sources
of published examples against which
I can test the code and any other comments
about either the documentation or the code.

The problem of meta--analysis of $p$--values is of course
not completely unconnected with the more general issue
of simultaneous statistical inference.

\subsection{Why and when to meta--analyse significance values}

The canonical way to meta--analyse a number of primary studies
uses estimates of effect sizes from each of them.
There are a large number of packages for
this purpose available from CRAN
and
described in the task view \url{http://CRAN.R-project.org/view=MetaAnalysis}.
However sometimes
the only available information may be $p$--values
especially when some of the primary
studies were published a long time ago or were published
in sources which were less rigorous about insisting on effect
sizes.
The methods outlined here are designed for this
eventuality.
The situation may also arise that some of the studies
can be combined in a conventional meta--analysis
using effect sizes but there
are many others which cannot and in that case the conventional
meta--analysis of the
subset of studies which do have effect sizes
may usefully be supplemented by an overall
analysis of the $p$--values.

Just for the avoidance of doubt,
if each study has produced a proportion and the goal is
to synthesise them to a common estimate or analyse the
differences between them then the standard methods
are appropriate not the ones outlined here.
The $p$--values in this document are significance levels.

The methods are referred to by the name of the
function in \func{metap}.
Table \ref{funcs} shows other
descriptions of each method.

\begin{table}[htbp]
\begin{tabular}{lll}
Function name & \multicolumn{2}{c}{Description(s)} \\[1ex]
 & \multicolumn{1}{c}{Eponym} \\
\func{invchisq} & Lancaster's method & Inverse chi square \\
\func{invt} & & Inverse t \\
\func{logitp} & & Logistic\\
\func{meanp} \\
\func{meanz} \\
\func{maximump} \\
\func{minimump} & Tippett's method \\
\func{sumlog} & Fisher's method & Chi square (2 df)\\
\func{sump} & Edgington's method & Uniform\\
\func{sumz} & Stouffer's method & Normal\\
\func{truncated} & Truncated Fisher & rank--truncated\\
\func{votep} \\
\func{wilkinsonp} & Wilkinson's method \\
\end{tabular}
\caption{Methods considered in this document}
\label{funcs}
\end{table}

\section{Preparation for meta--analysis of $p$--values}

\subsection{Preliminaries}

I assume you have installed \textsf{R} and \pkg{metap}.
You then need to load the package.
<<>>=
library(metap)
@

\subsection{Directionality}

It is usual to have a directional hypothesis, for
instance that treatment is better than control.
For the methods described here a necessary preliminary
is to ensure that all the $p$--values
refer to the same directional hypothesis.
If the value from the primary study is two--sided it needs to
be converted.
This is not simply a matter of halving the quoted $p$--value
as values in the opposite direction need to be reversed.
A convenience function \func{two2one} is provided for this.

{\codefont
<<>>=
pvals <- c(0.1, 0.1, 0.9, 0.9, 0.9, 0.9)
istwo <- c(TRUE,  FALSE, TRUE, FALSE, TRUE, FALSE)
toinvert <- c(FALSE, TRUE, FALSE, FALSE, TRUE, TRUE)
two2one(pvals, two = istwo, invert = toinvert)
@
}
Note in particular the way in which $0.9$ is
converted under the different scenarios.

\subsection{Plotting}
It would be a wise precaution to examine the $p$--values
graphically or otherwise before subjecting them to further analysis.
A separate vignette discusses the range of plots
available in the package so here we just show the
most simple.
As our example we use data from studies of validity of
student ratings of their instructors \citep{becker94}.
      
The \func{plotp} provides a Q--Q plot of
the $p$--values to detect departure from
the uniform distribution.
An example is shown in Figure \ref{plotp}.
The line is the line of exact fit to the
reference distribution, the uniform.
The polygon is a simultaneous confidence region
such that if points lie outside it we
can reject the null hypothesis that the points
are drawn iid from a uniform.

<<fig=TRUE,label=plotp,include=FALSE>>=
data(dat.metap)
validity <- dat.metap$validity$p
plotp(validity)
@

\mygraph{metap-plotp}{Q--Q plot from \func{plotp}}{plotp}

This plot is more informative than
a simple printout.
Note that this enhanced plot is only available from
version 1.8 of \func{metap}.
Previous versions usd a simpler plot.

{\codefont
<<>>=
print(validity)
@
}

\subsection{Reporting problems in the primary studies}

Another issue is what to do with studies which have simply reported
on whether a conventional level of significance like 0.05
was achieved or not.
If the exact associated $p$ cannot be derived from the
statistics quoted in the primary source then the value
of the level achieved, in this case 0.05, can be used
although this may be conservative.
Studies which simply report not significant could be included
as having $p=1$ (or $p=0.5$
if it is known that the direction was right)
although this is very conservative.
The theory of handling $p$--values which have been truncated like this
has been developed by \citet{zaykin02} and
\func{truncated} provides a convenience wrapper for
two methods available in other CRAN
packages.

\section{Using the methods}

All the methods in the package take as their first
argument the vector of $p$--values.
To use Fisher's method as an example:

<<>>=
sumlog(validity)
@

This confirms what was indeed obvious from the plot
that the null hypothesis that these are drawn from
a uniform distribution can be rejected.

A few require extra information.
Those which rely on inverse transformations
often need a vector of degrees of freedom.
Currently this applies to \func{invchisq} and \func{invt}.
Stouffer's method in \func{sumz}
optionally uses weights
if a vector of weights is provided.
Most of the methods
(\func{invchisq, invt, logitp, meanz, sumlog, sumz, wilkinsonp})
allow as an option to return the
logarithm of the $p$--value which may be useful if
it is expected that the return value will be very small.
A smaller number (\func{invchisq, invt, sumlog})
allow for input of log $p$--values.

\section{Miscellanea}

\begin{description}
\item[Extractor functions]
The standard \code{print} and \code{plot} methods
are provided.
\item[Omnibus function]
A function \func{allmetap} is provided to simultaneously
perform a number of the other functions.
It may be useful to show how they give different results
on the same data.
\item[Reading]
An annotated bibliography is provided by \citet{cousins08}
\end{description}

\bibliography{metap}
\bibliographystyle{plainnat}
\end{document}

\documentclass[12pt]{article}
\usepackage{amsmath,amssymb}
%\usepackage{mydef2}
\usepackage[round]{natbib}
\usepackage{parskip,url}
\usepackage{graphicx,subfig}
\setlength{\topmargin}{0cm}
\addtolength{\textheight}{2cm}
%\lhead{}
%\input{title}
%\VignetteIndexEntry{Introduction to the metap package}
\title{Introduction to the \pkg{metap} package}
\author{Michael Dewey}
\newcommand{\pkg}[1]{\texttt{#1}}
\newcommand{\func}[1]{\texttt{#1}}
\newcommand{\code}[1]{\texttt{#1}}
\newcommand{\codefont}{\footnotesize}
\newcommand{\mygraph}[3]{%
\begin{figure}[htbp]
\includegraphics[height=6cm,width=10cm]{#1}
\caption{#2}
\label{#3}
\end{figure}
}
\newcommand{\twograph}[8]{%
\begin{figure}[htbp]
\subfloat[#2\label{#3}]{\includegraphics[height=6cm,width=7cm]{#1}}%
\subfloat[#5\label{#6}]{\includegraphics[height=6cm,width=7cm]{#4}}
\caption{#7}
\label{#8}
\end{figure}
}
\begin{document}
\maketitle

\section{Introduction}

\subsection{What is this document for?}

This document describes some methods for the meta--analysis of
$p$--values
(significance values)
and their implementation
in the package \pkg{metap}.
I welcome feedback about sources
of published examples against which
I can test the code and any other comments
about either the documentation or the code.

The problem of meta--analysis of $p$--values is of course
not completely unconnected with the more general issue
of simultaneous statistical inference.

\subsection{Why and when to meta--analyse significance values}

The canonical way to meta--analyse a number of primary studies
uses estimates of effect sizes from each of them.
There are a large number of packages for
this purpose available from CRAN
and
described in the task view \url{http://CRAN.R-project.org/view=MetaAnalysis}.
However sometimes
the only available information may be $p$--values
especially when some of the primary
studies were published a long time ago or were published
in sources which were less rigorous about insisting on effect
sizes.
The methods outlined here are designed for this
eventuality.
The situation may also arise that some of the studies
can be combined in a conventional meta--analysis
using effect sizes but there
are many others which cannot and in that case the conventional
meta--analysis of the
subset of studies which do have effect sizes
may usefully be supplemented by an overall
analysis of the $p$--values.

Just for the avoidance of doubt,
if each study has produced a proportion and the goal is
to synthesise them to a common estimate or analyse the
differences between them then the standard methods
are appropriate not the ones outlined here.
The $p$--values in this document are significance levels.

The methods are referred to by the name of the
function in \func{metap}.
Table \ref{funcs} shows other
descriptions of each method.

\begin{table}[htbp]
\begin{tabular}{lll}
Function name & \multicolumn{2}{c}{Description(s)} \\[1ex]
 & \multicolumn{1}{c}{Eponym} \\
\func{invchisq} & Lancaster's method & Inverse chi square \\
\func{invt} & & Inverse t \\
\func{logitp} & & Logistic\\
\func{meanp} \\
\func{meanz} \\
\func{maximump} \\
\func{minimump} & Tippett's method \\
\func{sumlog} & Fisher's method & Chi square (2 df)\\
\func{sump} & Edgington's method & Uniform\\
\func{sumz} & Stouffer's method & Normal\\
\func{truncated} & Truncated Fisher & rank--truncated\\
\func{votep} \\
\func{wilkinsonp} & Wilkinson's method \\
\end{tabular}
\caption{Methods considered in this document}
\label{funcs}
\end{table}

\section{Preparation for meta--analysis of $p$--values}

\subsection{Preliminaries}

I assume you have installed \textsf{R} and \pkg{metap}.
You then need to load the package.
<<>>=
library(metap)
@

\subsection{Directionality}

It is usual to have a directional hypothesis, for
instance that treatment is better than control.
For the methods described here a necessary preliminary
is to ensure that all the $p$--values
refer to the same directional hypothesis.
If the value from the primary study is two--sided it needs to
be converted.
This is not simply a matter of halving the quoted $p$--value
as values in the opposite direction need to be reversed.
A convenience function \func{two2one} is provided for this.

{\codefont
<<>>=
pvals <- c(0.1, 0.1, 0.9, 0.9, 0.9, 0.9)
istwo <- c(TRUE,  FALSE, TRUE, FALSE, TRUE, FALSE)
toinvert <- c(FALSE, TRUE, FALSE, FALSE, TRUE, TRUE)
two2one(pvals, two = istwo, invert = toinvert)
@
}
Note in particular the way in which $0.9$ is
converted under the different scenarios.

<<echo = FALSE>>=
data(validity)
@

\subsection{Plotting}
{\codefont
<<>>=
print(validity)
@
}
It would be a wise precaution to examine the $p$--values
graphically or otherwise before subjecting them to further analysis.
Two functions are provided for this purpose: \func{plotp}
and \func{schweder}.

\subsubsection{Plotting using \func{plotp}}

The \func{plotp} provides a Q--Q plot of
the $p$--values to detect departure from
the uniform distribution.
An example is shown in Figure \ref{plotp}.
The standard line through through the quartiles
is superimposed.
This is the function which is called when the \func{plot}
method is used on an object returned by any of the
meta--analysis functions.

<<fig=TRUE,label=plotp,include=FALSE>>=
plotp(validity)
@

\mygraph{metap-plotp}{Q--Q plot from \func{plotp}}{plotp}

\subsubsection{Plotting using \func{schweder}}

A function \func{schweder} provides plots
with a variety of informative lines
superimposed.
It plots the ordered $p$--values, $p_{[i]}$, against
$i$.
Although the original motivation
for the plot is \citet{schweder82}
the function uses a different choice of axes due
to
\citet{benjamini00}.
We will use an example dataset on the validity of student
ratings quoted in \citet{becker94}.
Figure \ref{simple} shows the plot from \func{schweder}
which is the same as from \func{plotp} but without the line.

<<fig=TRUE,label=simple,include=FALSE>>=
schweder(validity)
@

\func{schweder} also offers the possibility of drawing one of 
a number of straight line summaries.
The three possible straight line summaries are
shown in Figure \ref{withlines} and are:

\begin{itemize}
\item
the lowest slope line of Benjaimin and Hochberg
which is drawn by default
as solid,
\item
a least squares line drawn passing through the point
$k+1, 1$
and using a specified fraction of the points
which is drawn by default as dotted,
\item
a line with user specified
intercept and slope
which is drawn by default as dashed.
\end{itemize}

<<fig=TRUE,label=withlines,include=FALSE>>=
schweder(validity, drawline = c("bh", "ls", "ab"),
   ls.control = list(frac = 0.5), ab.control = list(a = 0, b = 0.01))
@

\twograph{metap-simple}{Simple graph}{simple}{metap-withlines}{With lines}{withlines}{Output from schweder}{schweder}
\subsection{Reporting problems in the primary studies}

Another issue is what to do with studies which have simply reported
on whether a conventional level of significance like 0.05
was achieved or not.
If the exact associated $p$ cannot be derived from the
statistics quoted in the primary source then the value
of the level achieved, in this case 0.05, can be used
although this may be conservative.
Studies which simply report not significant could be included
as having $p=1$ (or $p=0.5$
if it is known that the direction was right)
although this is very conservative.
The theory of handling $p$--values which have been truncated like this
has been developed by \citet{zaykin02} and
\func{truncated} provides a convenience wrapper for
two methods available in other CRAN
packages.

\section{Using the methods}

All the methods in the package take as their first
argument the vector of $p$--values.
To use Fisher's method as an example:

<<>>=
sumlog(validity)
@

A few require extra information.
Those which rely on inverse transformations
often need a vector of degrees of freedom.
Currently this applies to \func{invchisq} and \func{invt}.
Stouffer's method in \func{sumz}
optionally uses weights
if a vector of weights is provided.

\section{Miscellanea}

\begin{description}
\item[Extractor functions]
The standard \code{print} and \code{plot} methods
are provided.
\item[Reading]
An annotated bibliography is provided by \citet{cousins08}
\end{description}

\bibliography{metap}
\bibliographystyle{plainnat}
\end{document}

\name{sumz}
\alias{sumz}
\alias{Stouffer's method}
\alias{Sum of normals method}
\alias{print.sumz}
\title{Combine p-values using the sum of z (Stouffer's) method}
\description{Combine \eqn{p}-values using the sum z method}
\usage{
sumz(p, weights = NULL, data = NULL, subset = NULL, na.action = na.fail)
\method{print}{sumz}(x, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{p}{\sigvec}
  \item{weights}{A vector of weights}
  \item{data}{Optional data frame containing variables}
  \item{subset}{Optional vector of logicals to specify a subset of the \eqn{p}-values}
  \item{na.action}{A function indicating what should happen when data
      contains NAs}
  \item{x}{An object of class \sQuote{\code{sumz}}}
  \item{...}{Other arguments to be passed through}
}
\details{
Defined as \deqn{\frac{\sum_{i=1}^k  w_i z(p_i)}{\sqrt {\sum_{i=1}^k  w_i ^ 2}}}{sum (w * z(p)) / sqrt(sum (w * w))}
is a \eqn{z} where \eqn{k} is the number of studies
and \eqn{w} are the weights \insertCite{stouffer49}{metap}.
By default the weights are equal.
In the absence of effect sizes (in which case a method for
combining effect sizes would be more appropriate anyway)
best results are believed to be obtained with weights
proportional to the square root of the sample sizes
\insertCite{zaykin11}{metap}

\insertNoCite{becker94}{metap}
\insertNoCite{rosenthal78}{metap}

\ltlt{two} If the omitted \eqn{p} values had supplied weights
a further warning is issued.

\plotmethod
}
\value{An object of class \sQuote{\code{sumz}} and
\sQuote{\code{metap}}, a list with entries
  \item{z}{Transformed sum of \eqn{z} values}
  \item{p}{Associated \eqn{p}-value}
  \item{validp}{The input vector with illegal values removed}
  \item{weights}{The weight vector corresponding to \code{validp}}
}
\references{
\insertAllCited{}
}
\author{Michael Dewey}
\seealso{
See also \code{\link{plotp}}
}
\examples{
data(teachexpect)
sumz(teachexpect) # z = 2.435, p = 0.0074, from Becker
data(beckerp)
sumz(beckerp) # z = 1.53, NS, from Beckerp
data(rosenthal)
sumz(rosenthal$p) # 2.39, p = 0.009
sumz(p, df, rosenthal) # 3.01, p = 0.0013
data(validity)
sumz(validity) # z = 8.191, p = 1.25 * 10^{-16}
}
\keyword{ htest }

\name{two2one}
\alias{two2one}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Convert two-sided p-values to one-sided}
\description{Convert two-sided \eqn{p}-values to one-sided}
\usage{
two2one(p, two = NULL, invert = NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{p}{A vector of p-values}
  \item{two}{A logical vector defining which
   \eqn{p}-values are two-sided and to be converted}
  \item{invert}{A logical vector defining which
  \eqn{p}-values are to be inverted}
}
\details{The \eqn{p}-values supplied to the other functions should be
one-sided and all in the same direction.
This convenience function will convert two-sided to
one-sided and invert as necessary.
By default it is assumed that all \eqn{p}-values are two-sided
and the function converts them all to one-sided.
Use \code{invert} to specify if some of the tests
resulted in outcomes in the other direction.
So for example a value of 0.05 will be converted to 0.025 unless
\code{invert} is TRUE in which case it is converted to 0.975.}
\value{A vector of one-sided p-values all in the correct direction
}
\author{Michael Dewey}

\examples{
data(rosenthal)
twop <- with(rosenthal, (pt(t, df)))
}
\keyword{ manip }

\name{invchisq}
\alias{invchisq}
\alias{Lancaster's method}
\alias{print.invchisq}
\title{Combine p values using inverse chi squared method}
\description{Combine \eqn{p}-values by the
inverse chi-squared method,
also known as Lancaster's method,
}
\usage{
invchisq(p, k, data = NULL, subset = NULL, na.action = na.fail)
\method{print}{invchisq}(x, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{p}{\sigvec}
  \item{k}{A vector of degrees of freedom to use, see details}
  \item{data}{Optional data frame containing variables}
  \item{subset}{Optional vector of logicals to specify a subset of the \eqn{p}-values}
  \item{na.action}{A function indicating what should happen when data
      contains NAs}
  \item{x}{An object of class \sQuote{\code{invchisq}}}
  \item{...}{Other arguments to be passed through}
}
\details{
Defined as
\deqn{\sum_{i=1}^n \chi^2_{k_i}(p_i) > \chi^2_{\sum{k_i}}(\alpha)}{\sum \chi^2_k(p) > \chi^2_\sum{k}(\alpha)}

If \code{k} is a single value it is used for all the \code{p}
of which there are \code{n}.
If any value of \eqn{k_i\le0} then the corresponding \eqn{p_i}
is not included.

\ltle{two}

\plotmethod
\nocancel
}
\value{
An object of class \sQuote{\code{invchisq}} and \sQuote{\code{metap}},
a list with entries
  \item{chisq}{Value of chi-squared statistic}
  \item{df}{Associated degrees of freedom}
  \item{p}{Associated p-value}
  \item{validp}{The input vector with the illegal values removed}
}
\references{
\insertRef{becker94}{metap}

\insertRef{lancaster49}{metap}
}
\author{
Michael Dewey
}
\seealso{
See also \code{\link{sumlog}} and \code{\link{plotp}}
}
\examples{
data(beckerp)
invchisq(beckerp, 2) # same as sumlog
invchisq(c(0.999, 0.999, 0.001, 0.001), 4)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ htest }

\name{albatros}
\alias{albatros}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Albatros plot}
\description{Produces the albatros plot of Harrison et al\loadmathjax}
\usage{
albatros(p, n, axes = list(xlimit = NULL, ylimit = NULL,
      lefttext = NULL, righttext = NULL, xlas = 3, ylas = 2),
   contours = list(type = NULL, contvals = NULL, ltys = NULL, contlabs = NULL),
   plotpars = list(yscale = "sqrt", yfun = NULL, pchs = NULL, cols = NULL), ...)
}
\arguments{
  \item{p}{A vector of \mjseqn{p} values}
  \item{n}{A vector of sample sizes}
  \item{axes}{A list containing
     \itemize{
     \item \code{xlimit} a single value, the limit
     for the x-axis which is used symmetrically
     \item \code{ylimit} a vector of length 2, limits for the y-axis
     \item \code{lefttext} character, to label below plot
     \item \code{righttext} character, to label below plot
     \item \code{xlas} numeric, las value for x-axis
     \item \code{ylas} numeric, las value for y-axis
     }
  }
  \item{contours}{A list containing
     \itemize{
     \item \code{type} what effect size to use, a character one of "corr",
        "or", "smd"
     \item \code{contvals} a vector, what values to draw the contours at
     \item \code{ltys} a vector, what line types to use
     \item \code{contlabs} logical, draw the labels of the countours?
     }
  }
  \item{plotpars}{A list containing
      \itemize{
      \item \code{yscale} Scaling for y-axis, See details
      \item \code{yfun} Optional user function for scaling y-axi
      \item \code{pchs} Symbols to be used
      \item \code{cols} Colours for points
      }
  }
  \item{\dots}{Arguments to be passed through to \code{plot}}
}
\details{Plots \mjseqn{n}, possibly
transformed, against trnsformed \mjseqn{p} and draws contours of
constant effect size.
\insertNoCite{harrison17}{metap}

The \mjseqn{p} value scale on the x-axis is a logarithmic one
but folded at the null value so that the extreme values represent
small probabilities in each direction.
The scale for the y-axis is user selectable.
The original scale in the Stata version is
\mjeqn{(\log_{10}n)^2}{log(n, base = 10) ^ 2} which is obtained by
setting \code{yscale} to "classic"
but it is
also possible to use \mjeqn{\sqrt{n}}{sqrt(n)}
which is the default or \mjseqn{n} by setting
\code{yscale} to "n".
Setting \code{yscale} to "userfun" allows for the
user to supply a function for transformation.
The function (of a single
argument) should be given to the \code{userfun}
parameter.

If the dataset contains extreme values the
plot may be uninformative and the \code{xlimit}
and \code{ylimit} parameters my be helpful to clip
the plot.
If they are not specified the function tries
to choose suitable values.
The values for \code{xlas} and \code{ylas}
control the orientation of the tick labels
relative to the plot.
The default values are usually satisfactory
but if there are many ticks they may need
setting explicitly.

For the contours if the \code{type} parameter in \code{contours}
is NULL (the default) no contours will be plotted.
The options available are
\dQuote{smd} (standardised mean difference)
\dQuote{corr} (correlation)
\dQuote{or} (odds ratio).

The contours are labelled unless contlabs is set to \code{FALSE}.
The function tries to position these so they are not obscured
by the contour lines but if that is unsatisfactory then
it is best to suppress them and position them manually
or use a legend to identify line types.

The \code{pch} parameter may either be a single value
or a vector of the same length as \code{p}.
It may contain anything which is legal as input to
the parameter of the same name in the graphics
library and will be used for the plotted points.
The \code{cols} parameter works similarly.

The values of \code{lefttext} and \code{righttext} are
used to produce labels under the \mjseqn{x}-axis and
are placed level with the label.
By default they are not produced.

The return values from the function may be useful if further
annotations are required as they give the axis limits in
plotting units.
These are what are needed for the arguments to
\code{legend} for instance.
Note that zero or one are not valid \mjseqn{p}-values and
are silently removed and values of
\mjseqn{n} less than 2
are not plotted.
}
\value{Returns
a list containing
  \item{xlims}{The limits for the x-axis in plotting units}
  \item{ylims}{The limits for the y-axis in plotting units}
  \item{validp}{The valid values for p}
  \item{validn}{The valid values for n}
}
\references{
\insertAllCited{}
}
\author{Michael Dewey}
%\note{The docontours function is for internal use only}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{For plotting parameters
see \code{\link{par}}
}
\examples{
data(dat.metap)
validity <- dat.metap$validity
fit.v <- albatros(validity$p, validity$n,
   contours = list(type = "corr", contvals = c(0.25, 0.5, 0.8), ltys = 1:3),
      axes = list(ylimit = c(1,200),  lefttext = "Negative correlation",
         righttext = "Positive correlation"),
   main = "Validity")
#
fit.z <- albatros(dat.metap$zhang$p, dat.metap$zhang$n,
   contours = list(type = "smd", contvals = c(0.25, 0.5, 1), ltys = 1:3),
   plotpars = list(pchs = letters[unclass(dat.metap$zhang$phase)]),
   axes = list(lefttext = "Favours control", righttext = "Favours exercise"),
   main = "Zhang"
   )
}
\keyword{hplot}

\newcommand{\nocancel}{Inspection of the distribution of \mjseqn{p}-values is highly recommended as extreme values in opposite directions do not cancel out. See last example. This may not be what you want.}
\newcommand{\plotmethod}{The plot method for class \sQuote{\code{metap}} calls \code{\link{plotp}} on the valid \eqn{p}-values.}
\newcommand{\sigvec}{A vector of significance values}
\newcommand{\ltle}{The values of \mjseqn{p_i} should be such that \mjteqn{0 < p_i\le 1}{0 \lt p_i \le 1}{0 < p_i\le 1} and a warning is given if that is not true. A warning is given if, possibly as a result of removing illegal values, fewer than #1 values remain and the return values are set to NA.}
\newcommand{\ltlt}{The values of \mjseqn{p_i} should be such that \mjteqn{0 < p_i < 1}{0 \lt p_i \lt 1}{0 < p_i < 1}and a warning is given if that is not true. A warning is given if, possibly as a result of removing illegal values, fewer than #1 values remain and the return values are set to NA.}
\newcommand{\lele}{The values of \mjseqn{p_i} should be such that \mjseqn{0\le p_i\le 1} and a warning is given if that is not true. A warning is given if, possibly as a result of removing illegal values, fewer than #1 values remain and the return values are set to NA.}
\newcommand{\logp}{Logical, if TRUE result is returned as log(p)}
\newcommand{\loginput}{Logical, if TRUE the input \mjseqn{p} values are assumed to be logged}

\name{invt}
\alias{invt}
\alias{print.invt}
\title{Combine p values using inverse t method}
\description{Combine p values using the inverse t method}
\usage{
invt(p, k, data = NULL, subset = NULL, na.action = na.fail)
\method{print}{invt}(x, ...)
}
\arguments{
  \item{p}{\sigvec}
  \item{k}{A vector of degrees of freedom to use, see details}
  \item{data}{Optional data frame containing variables}
  \item{subset}{Optional vector of logicals to specify a subset of the \eqn{p}-values}
  \item{na.action}{A function indicating what should happen when data
      contains NAs}
  \item{x}{An object of class \sQuote{\code{invt}}}
  \item{...}{Other arguments to be passed through}
}
\details{
Defined as 
\deqn{\frac{\sum_{i=1}^n t_{k_i}(p_i)}%
{\sqrt{\sum_{i=1}^n \frac{k_i}{k_i - 2}}}%
> z(\alpha)%
}{\sum t (p) /%
\sqrt{\sum k / (k - 2)}%
> z(\alpha)}

If \code{k} is a single value it is used for all the \code{p}
of which there are \code{n}.
If any value of \eqn{k_i\le2} then the corresponding \eqn{p_i}
is not included.

\ltle{two}

\plotmethod
}
\value{
An object of class \sQuote{\code{invt}} and \sQuote{\code{metap}},
a list with entries
  \item{z}{Value of z}
  \item{p}{Associated p-value}
  \item{validp}{The input vector with the illegal values removed}
}
\references{
\insertRef{becker94}{metap}
}
\author{Michael Dewey}

\seealso{
See also \code{\link{plotp}}
}

\examples{
data(beckerp)
invt(beckerp, 50)
}
\keyword{ htest }

\name{package-data}
\alias{beckerp}
\alias{cholest}
\alias{edgington}
\alias{naep}
\alias{mourning}
\alias{rosenthal}
\alias{teachexpect}
\alias{validity}
\docType{data}
\title{Example data}
\description{
The package contains the following datasets: \code{beckerp},
   \code{cholest}, \code{edgington}, \code{mourning}, \code{naep},
   \code{rosenthal}, \code{teachexpect}, and \code{validity}.
}
\usage{data(beckerp)
data(cholest)
data(edgington)
data(mourning)
data(naep)
data(rosenthal)
data(teachexpect)
data(validity)
}
\format{
  \describe{
    \item{\code{beckerp}}{
  A vector of length 5
}
    \item{\code{cholest}}{
  A vector of length 34
}
   \item{\code{edgington}}{
   A vector of length 7
}
    \item{\code{naep}}{A data frame with 34 observations on the following 2 variables.
  \describe{
    \item{\code{state}}{a factor with levels \code{AL}, \code{AR}, \code{AZ}, \code{CA}, \code{CO}, \code{CT}, \code{DE}, \code{FL}, \code{GA}, \code{HI}, \code{IA}, \code{ID}, \code{IN}, \code{KY}, \code{LA}, \code{MD}, \code{MI}, \code{MN}, \code{NC}, \code{ND}, \code{NE}, \code{NH}, \code{NJ}, \code{NM}, \code{NY}, \code{OH}, \code{OK}, \code{PA}, \code{RI}, \code{TX}, \code{VA}, \code{WI}, \code{WV}, \code{WY},}
    \item{\code{p}}{a numeric vector}
  }}
  \item{\code{mourning}}{ A data frame with 9 observations on the following 3 variables.
  \describe{
    \item{\code{stance}}{a factor with levels \code{No stand}, \code{Opponent}, \code{Supporter}}
    \item{\code{grade}}{a factor with levels \code{G11-12}, \code{G7-8}, \code{G9-10}}
    \item{\code{p}}{a numeric vector}
  }}
   \item{\code{rosenthal}}{
  A data frame with 5 observations on the following 3 variables.
  \describe{
    \item{\code{t}}{A numeric vector of values of \eqn{t}}
    \item{\code{df}}{a numeric vector of degrees of freedom}
    \item{\code{p}}{a numeric vector of one sided \eqn{p} values}
  }
  }
  \item{\code{teachexpect}}{A vector of length 19
   }
   \item{\code{validity}}{A vector of length 20
   }
}
}
\details{
\describe{
   \item{\code{beckerp}}{Hypothetical \eqn{p} values from \insertCite{becker94;textual}{metap}}
   \item{\code{cholest}}{\eqn{p}-values from studies of cholesterol
       lowering from \insertCite{sutton00;textual}{metap}}
   \item{\code{edgington}}{Hypothetical \eqn{p} values from \insertCite{edgington72a;textual}{metap}}
   \item{\code{mourning}}{Results from a study of mourning practices of Israeli
       youth from \insertCite{benjamini00;textual}{metap}}
   \item{\code{naep}}{Results on the National Assessment
      of Educational Progress from \insertCite{benjamini00;textual}{metap}}
   \item{\code{rosenthal}}{Hypothetical example from \insertCite{rosenthal78;textual}{metap}}
   \item{\code{teachexpect}}{\eqn{p}-values from studies of the effect of
      manipulating teacher expectancy from \insertCite{becker94;textual}{metap}}
   \item{\code{validity}}{\eqn{p}-values from studies of validity of
      student ratings from \insertCite{becker94;textual}{metap}}
   }
}

\references{
\insertAllCited{}
}

\keyword{datasets}

\name{metap-internal}
\alias{metap-internal}
\alias{docontours}
\alias{makexscale}
\alias{makeyscale}
\title{Internal functions}
\description{Functions for internal use}
\usage{
docontours(conttype, xvals, contvals, xtrans, ytrans,
   xlims, ylims, ltys, contlabs)
makeyscale(yscale, userfun)
makexscale(p)
}
\details{Not to be called by the user}
\keyword{internal}

\newcommand{\nocancel}{Inspection of the distribution of \eqn{p}-values is highly recommended as extreme values in opposite directions do not cancel out. See last example. This may not be what you want.}
\newcommand{\plotmethod}{The plot method for class \sQuote{\code{metap}} calls \code{\link{plotp}} on the valid \eqn{p}-values.}
\newcommand{\sigvec}{A vector of significance values}
\newcommand{\ltle}{The values of \eqn{p_i} should be such that \eqn{0<p_i\le 1} and a warning is given if that is not true. An warning is given if, possibly as a result of removing illegal values, fewer than #1 values remain and the return values are set to NA.}
\newcommand{\ltlt}{The values of \eqn{p_i} should be such that \eqn{0<p_i<1} and a warning is given if that is not true. An warning is given if, possibly as a result of removing illegal values, fewer than #1 values remain and the return values are set to NA.}
\newcommand{\lele}{The values of \eqn{p_i} should be such that \eqn{0\le p_i\le 1} and a warning is given if that is not true. An warnin is given if, possibly as a result of removing illegal values, fewer than #1 values remain and the return values are set to NA.}

\name{truncated}
\alias{truncated}
\alias{print.truncated}
\alias{summary.truncated}
\alias{plot.truncated}
\title{Truncated product method routine}
\description{Implements two procedures for truncated versions
of Fisher's method}
\usage{
truncated(p, ptrunc = NULL, rtrunc = NULL, ...)
\method{print}{truncated}(x, ...)
\method{summary}{truncated}(object, ...)
\method{plot}{truncated}(x, pparams = list(pchs = c(16, 1),
   pcols = c("black", "black")), ...)
}
\arguments{
  \item{p}{\sigvec}
  \item{ptrunc}{Value of \eqn{p} to truncate at. See Details}
  \item{rtrunc}{Which rank \eqn{p} to truncate at. See Details}
  \item{x}{An object of class \sQuote{\code{truncated}}}
  \item{object}{An object of class \sQuote{\code{truncated}}}
  \item{\dots}{Other arguments to be passed through}
  \item{pparams}{A list containing a vector \code{pchs} of plotting
     symbols and a vector \code{pcols} of colours}
}
\details{Uses routines from TFisher to implement
   truncated Fisher \insertCite{zaykin07,zhang18}{metap}
   and from mutoss to implement
   rank-truncated Fisher \insertCite{dudbridge03}{metap}

   The print method prints the statistic and its associated
   \eqn{p}-value. The summary method also prints how many
   of the input values were actually used. The plot method
   provides an index plot of the \eqn{p}-values.
   By default used \eqn{p}-values are plotted
   with a black filled circle and those not used with a black
   unfilled circle. The colours and plotting symbols can be changed
   in \code{pparams}. The first element of each vector is for the
   used \eqn{p}-values and the second for the unused ones.
}
\value{An object of class \sQuote{\code{truncated}}
a list with entries
  \item{p}{The resultant overall \eqn{p}-value}
  \item{stat}{The test statistic}
  \item{validp}{The input vector with illegal values removed}
  \item{ptrunc}{The \eqn{p}-value used for truncation}
  \item{trunc}{The  rank value used for truncation}
  \item{n}{A vector containing \code{incl}, the number
     of valid values included, \code{excl}, the number of valid
     values not used}
  \item{method}{The method used}
}
\note{
   Only one of \code{ptrunc} or \code{rtrunc} is returned
   as appropriate. If the function encounters an error
   it returns NA for the value of \code{p} and gives
   a warning message.
   The effect of specifying both \code{ptrunc}
   and \code{rtrunc} is undefined.
}
\references{
\insertAllCited{}
}
\author{
Michael Dewey
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{ranktruncated}}, \code{\link{stat.tpm}}, and \code{\link{p.tpm}}
}
\examples{
data(validity)
truncated(validity, ptrunc = 0.5)
truncated(validity, rtrunc = 5)
}
\keyword{htest1}

\name{meanz}
\alias{meanz}
\alias{print.meanz}
\title{Combine p values using the mean z method}
\description{Combines p values using the mean of z
method\loadmathjax}
\usage{
meanz(p, log.p = FALSE)
\method{print}{meanz}(x, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{p}{\sigvec}
  \item{log.p}{\logp}
  \item{x}{An object of class \sQuote{\code{meanz}}}
  \item{...}{Other arguments to be passed through}
}
\details{
Let \mjdeqn{\bar{z} = \sum_{i=1}^k \frac{z(p_i)}{k}}{bar{z} = sum(z(p) / k)}
and \mjdeqn{s_{\bar{z}} = \frac{s_z}{\sqrt{k}}}{s_{bar{z}} = s_z / sqrt k }
Defined as
\mjdeqn{%
\frac{\bar{z}}{s_{\bar{z}}} > t_{k-1}(\alpha)
}{((bar{z} / s_bar{z}) > t_{k-1}(alpha)}

\lele{two}
As can be seen if all the \mjseqn{p_i} are equal or close
to equal this gives a \mjeqn{t=\pm\infty}{t=+-infty} leading to
a returned value of 0 or 1. 
A set of \mjseqn{p} values with small variance will
necessarily give a large value
for \mjdeqn{\frac{\bar{z}}{s_{\bar{z}}}}{((bar{z} / s_bar{z})}
and hence a small
\mjseqn{p} value which may be smaller
than that for another set all of whose primary values are less
than any in the first set.
See examples for a demonstration.

\plotmethod
}
\value{An object of class \sQuote{\code{meanz}} and
\sQuote{\code{metap}}, a list with entries
  \item{z}{The value of the mean \mjseqn{z} statistic}
  \item{p}{The associated \mjseqn{p} value}
  \item{validp}{The input vector with illegal values removed}
}
\references{
\insertRef{becker94}{metap}
}
\author{Michael Dewey}

\seealso{
See also \code{\link{plotp}}
}

\examples{
data(dat.metap)
beckerp <- dat.metap$beckerp
meanz(beckerp)
meanz(c(0.1, 0.2))  # greater than next example
meanz(c(0.3, 0.31)) # less than above
all.equal(exp(meanz(beckerp, log.p = TRUE)$p), meanz(beckerp)$p)
}
\keyword{ htest }

\name{allmetap}
\alias{allmetap}
\alias{print.allmetap}
\title{Carry out all or some of the methods in the package}
\description{Enables all or a selected number of the package
methods to be executed on the same dataset}
\usage{
allmetap(p, method = NULL)
\method{print}{allmetap}(x, digits = 5, ...)
}
\arguments{
  \item{p}{A vector of \eqn{p}-values}
  \item{method}{A vector of character containing names of methods or \sQuote{\code{all}}}
  \item{x}{An object of class \sQuote{\code{allmetap}}}
  \item{digits}{How many digits to print?}
  \item{...}{Other arguments to be passed through}
 
}
\details{The function uses each of the methods specified
   on the data-set in \code{p}. If method contains \sQuote{\code{all}}
   then this over--rides any specific method(s) named.
}
\value{Returns an object of class
\sQuote{\code{allmetap}} and \sQuote{\code{data.frame}} containing columns
  \item{p}{The value of p returned}
  \item{valid}{The number of valid p values used for that method}
  \item{eponym}{The eponym, if any, for the method}
   The row names are the names of the methods used.
   If any of the methods cannot be applied to that data--set NA are
   returned for \code{p} and \code{valid}.
  }
\author{Michael Dewey}
\examples{
data(beckerp)
allmetap(beckerp, method = "all")
allmetap(beckerp, method = c("sumz", "sumlog"))
}
\keyword{htest}

\name{details}
\alias{details}
\docType{data}
\title{
Template for use in function allmetap
}
\description{
Provides a template for the output of allmetap}
\usage{details}
\format{
  A data frame with 8 observations on the following 2 variables.
  \describe{
    \item{\code{funcs}}{a character vector}
    \item{\code{eponyms}}{a character vector}
  }
}
\keyword{internal}

\name{sumlog}
\alias{sumlog}
\alias{Fisher's method}
\alias{chi-square (2) method}
\alias{print.sumlog}
\title{Combine p-values by the sum of logs (Fisher's) method
}
\description{Combine \eqn{p}-values by the sum of logs method,
also known as Fisher's method, and sometimes as the
chi-square (2) method.}
\usage{
sumlog(p)
\method{print}{sumlog}(x, ...)
}
\arguments{
  \item{p}{\sigvec}
  \item{x}{An object of class \sQuote{\code{sumlog}}}
  \item{...}{Other arguments to be passed through}
}
\details{
The method relies on the fact that
\deqn{\sum_{i=1}^k - 2 \log p_i}{sum (-2 log p)}
is a chi-squared with \eqn{2 k}{2 * k} df where \eqn{k} is the number
of studies \insertCite{fisher25}{metap}.
\insertNoCite{becker94}{metap}
\insertNoCite{rosenthal78}{metap}
\insertNoCite{sutton00}{metap}

\ltle{two}

\plotmethod
\nocancel
}
\value{
An object of class \sQuote{\code{sumlog}} and \sQuote{\code{metap}},
a list with entries
  \item{chisq}{Value of chi-squared statistic}
  \item{df}{Associated degrees of freedom}
  \item{p}{Associated \eqn{p}-value}
  \item{validp}{The input vector with the illegal values removed}
%% ...
}
\references{
\insertAllCited{}
}

\author{Michael Dewey}

\seealso{
See also \code{\link{plotp}}
}
\examples{
data(teachexpect)
sumlog(teachexpect) # chisq = 69.473, df = 38, p = 0.0014, from Becker
data(beckerp)
sumlog(beckerp) # chisq = 18.533, df = 10, sig
data(rosenthal)
sumlog(rosenthal$p) # chisq = 22.97, df = 10, p = 0.006 one sided
data(cholest)
sumlog(cholest) # chisq = 58.62, df = 68, p = 0.78
data(validity)
sumlog(validity) # chisq = 159.82, df = 40, p = 2.91 * 10^{-16}
sumlog(c(0.0001, 0.0001, 0.9999, 0.9999)) # is significant
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ htest }

\name{sump}
\alias{sump}
\alias{Edgington's method}
\alias{print.sump}
\title{Combine p-values using the sum of p (Edgington's) method}
\description{Combine \eqn{p}-values using the sum p method}
\usage{
sump(p)
\method{print}{sump}(x, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{p}{\sigvec}
  \item{x}{An object of class \sQuote{\code{sump}}}
  \item{...}{Other arguments to be passed through}
}
\details{
Defined as
\deqn{\frac{(\sum_{i=1}^k p_i)^k}{k!}
- {k - 1 \choose 1}\frac{(\sum_{i=1}^k  p_i - 1)^k}{k!}
+ {k - 2 \choose 2}\frac{(\sum_{i=1}^k  p_i - 2)^k}{k!} ...}{%
((sum p) ^ k) / k! - (k-1)C(1) ((sum p - 1) ^ k) / k! + (k-2)C(2) ((sum p - 2) ^ k) / k! ...}
where there are \eqn{k} studies and the series continues until the numerator
becomes negative \insertCite{edgington72a}{metap}.

Some authors use a simpler version
\deqn{\frac{(\sum_{i=1}^k  p_i)^k}{k!}}{((sum p) ^ k) / k!}
but this can be very conservative when
\eqn{\sum_{i=1}^k  p_i > 1}{sum p > 1}.
There seems no particular need to use this method but
it is returned as the value of \code{conservativep}
for use in checking published values.

\lele{two} A warning is given when the internal calculations
are likely to have been subject to numerical error
and an alternative method should be used to check
the result.

\plotmethod
}
\value{An object of class \sQuote{\code{sump}} and
\sQuote{\code{metap}}, a list with entries
  \item{p}{The transformed sum of the \eqn{p}-values}
  \item{conservativep}{See details}
  \item{validp}{The input vector with illegal values removed}
}
\references{

\insertAllCited{}

}
\author{Michael Dewey}
\seealso{
See also \code{\link{plotp}}
}
\examples{
data(edgington)
sump(edgington) # p = 0.097
}
\keyword{ htest }

\name{dat.metap}
\alias{dat.metap}
 \alias{beckerp}
 \alias{cholest}
 \alias{edgington}
 \alias{naep}
 \alias{mourning}
 \alias{rosenthal}
 \alias{teachexpect}
 \alias{validity}
\docType{data}
\title{Example data}
\description{
The package contains the following datasets: \code{beckerp},
   \code{cholest}, \code{edgington}, \code{mourning}, \code{naep},
   \code{rosenthal}, \code{teachexpect}, \code{validity},
   and \code{zhang}.
\loadmathjax}
\usage{data(dat.metap)
}
\format{A list with the following elements:
  \describe{
    \item{\code{beckerp}}{
  A vector of length 5 hypothetical \mjseqn{p} values
}
    \item{\code{cholest}}{
  A data frame with 34 observations on the following 5 variables.
  \describe{
    \item{\code{ntreat}}{A numeric vector of numbers in the treated group}
    \item{\code{ncontrol}}{A numeric vector of number in the control group}
    \item{\code{dtreat}}{A numeric vector of number of deaths in the treated group}
    \item{\code{dcontrol}}{A numeric vector of number of deaths in the control group}
    \item{\code{p}}{a numeric vector of one sided \mjseqn{p} values}
  }
}
   \item{\code{edgington}}{
   A vector of length 7 hypothetical \mjseqn{p} values
}
    \item{\code{naep}}{A data frame with 34 observations on the following 2 variables.
  \describe{
    \item{\code{state}}{a factor with levels \code{AL}, \code{AR}, \code{AZ}, \code{CA}, \code{CO}, \code{CT}, \code{DE}, \code{FL}, \code{GA}, \code{HI}, \code{IA}, \code{ID}, \code{IN}, \code{KY}, \code{LA}, \code{MD}, \code{MI}, \code{MN}, \code{NC}, \code{ND}, \code{NE}, \code{NH}, \code{NJ}, \code{NM}, \code{NY}, \code{OH}, \code{OK}, \code{PA}, \code{RI}, \code{TX}, \code{VA}, \code{WI}, \code{WV}, \code{WY},}
    \item{\code{p}}{a numeric vector}
  }}
  \item{\code{mourning}}{ A data frame with 9 observations on the following 3 variables.
  \describe{
    \item{\code{stance}}{a factor with levels \code{No stand}, \code{Opponent}, \code{Supporter}}
    \item{\code{grade}}{a factor with levels \code{G11-12}, \code{G7-8}, \code{G9-10}}
    \item{\code{p}}{a numeric vector of \mjseqn{p} values}
  }}
   \item{\code{rosenthal}}{
  A data frame with 5 observations on the following 3 variables.
  \describe{
    \item{\code{t}}{A numeric vector of values of \mjseqn{t}}
    \item{\code{df}}{a numeric vector of degrees of freedom}
    \item{\code{p}}{a numeric vector of one sided \mjseqn{p} values}
  }
  }
  \item{\code{teachexpect}}{A vector of length 19 hypothetical \mjseqn{p} values
   }
   \item{\code{validity}}{
   A data frame with 20 observations on the following 3 variables
  \describe{
    \item{\code{n}}{A numeric vector of sample sizes}
    \item{\code{r}}{a numeric vector of correlation coefficients}
    \item{\code{p}}{a numeric vector of one sided \mjseqn{p} values}
  }
   }
   \item{\code{zhang}}{
   A data frame with 22 observations on the following 11 variables
   \describe{
      \item{\code{study}}{character, the study names}
      \item{\code{smd}}{numeric, the standardised mean difference}
      \item{\code{lo}}{numeric, the lower confidence limit}
      \item{\code{hi}}{numeric, the upper confidence limit}
      \item{\code{ntreat}}{numeric, the treated sample size}
      \item{\code{ncont}}{numeric, the control sample size}
      \item{\code{n}}{numeric, the total sample size}
      \item{\code{phase}}{factor, what phase the patients were in:
         acute, healing, healed}
      \item{\code{sd}}{numeric, the calculated standard deviation}
      \item{\code{z}}{numeric, the calculated z}
      \item{\code{p}}{numeric, the probability associated with z}
   }
   }
}
}
\details{
\describe{
   \item{\code{beckerp}}{Hypothetical \mjseqn{p} values from \insertCite{becker94;textual}{metap}}
   \item{\code{cholest}}{\if{html}{\eqn{}}\mjseqn{p} values from trials
       of interventions for cholesterol
       lowering from \insertCite{sutton00;textual}{metap}}
   \item{\code{edgington}}{Hypothetical \mjseqn{p} values from \insertCite{edgington72a;textual}{metap}}
   \item{\code{mourning}}{Results from a study of mourning practices of Israeli
       youth following the assassination of Itzakh Rabin
       from \insertCite{benjamini00;textual}{metap}}
   \item{\code{naep}}{Results of mathematical achievment scores
      from the National Assessment
      of Educational Progress from \insertCite{benjamini00;textual}{metap}}
   \item{\code{rosenthal}}{Hypothetical example from \insertCite{rosenthal78;textual}{metap}}
   \item{\code{teachexpect}}{\if{html}{\eqn{}}\mjseqn{p}-values from studies of the effect of
      manipulating teacher expectancy on student IQ
      from \insertCite{becker94;textual}{metap}}
   \item{\code{validity}}{Data from studies of validity of
      student ratings of their instructors
      from \insertCite{becker94;textual}{metap}
      including correlations and sample sizes as well as \mjseqn{p}-values}
   \item{\code{zhang}}{Data from trials of exercise training for
      patients with cardiovascular disease from
      \insertCite{zhang16;textual}{metap}
   }
}
}
\author{Michael Dewey}

\note{The \mjseqn{p}-values in \code{cholest} have been re-calculated
from other data given in the book
and so are of higher accuracy than the ones
given in the book which are only to two decimal places.}

\references{
\insertAllCited{}
}
\examples{
data(dat.metap)
}
\keyword{datasets}

\name{wilkinsonp}
\alias{wilkinsonp}
\alias{maximump}
\alias{minimump}
\alias{Tippett's method}
\alias{print.wilkinsonp}
\alias{print.maximump}
\alias{print.minimump}
\title{Combine p-values using Wilkinson's method}
\description{Combine \eqn{p}-values using Wilkinson's method}
\usage{
wilkinsonp(p, r = 1, alpha = 0.05)
maximump(p, alpha = 0.05)
minimump(p, alpha = 0.05)
\method{print}{wilkinsonp}(x, ...)
\method{print}{maximump}(x, ...)
\method{print}{minimump}(x, ...)
}
\arguments{
  \item{p}{\sigvec}
  \item{r}{Use the \eqn{r}th  smallest \eqn{p} value}
  \item{alpha}{The significance level}
  \item{x}{An object of class \sQuote{\code{wilkinsonp}}
    or of class \sQuote{\code{maximump}}
    or of class \sQuote{\code{minimump}}}
  \item{...}{Other arguments to be passed through}
}
\details{
Wilkinson \insertCite{wilkinson51}{metap}
originally proposed his method in the context of
simultaneous statistical inference: the probability
of obtaining \eqn{r} or more significant statistics by
chance in a group of \eqn{k}.
The values are obtained from the Beta distribution, see
\code{\link{pbeta}}.

If \code{alpha} is greater than unity
it is assumed to be a percentage. Either values greater than 0.5 (assumed to
be confidence coefficient) or less than 0.5 are accepted.

\lele{two}

\code{maximump} and
\code{minimump} each provide a wrapper for \code{wilkinsonp}
for the special case when \eqn{r = \mathrm{length}(p)}{r = length(p)}
or \eqn{r=1} respectively and each has its own
print method.
The method of minimum \eqn{p} is also known as Tippett's method
\insertCite{tippett31}{metap}.
\insertNoCite{becker94}{metap}\insertNoCite{birnbaum54}{metap}


\plotmethod
\nocancel
}
\value{An object of class \sQuote{\code{wilkinsonp}}
and \sQuote{\code{metap}} or of class \sQuote{\code{maximump}}
and \sQuote{\code{metap}} or of class \sQuote{\code{minimump}}
and \sQuote{\code{metap}},
a list with entries
  \item{p}{The \eqn{p}-value resulting from the meta--analysis}
  \item{pr}{The \eqn{r}th smallest \eqn{p} value used}
   \item{r}{The value of \eqn{r}}
  \item{critp}{The critical value at which the \eqn{r}th value
      would have been significant for the chosen \code{alpha}}
  \item{validp}{The input vector with illegal values removed}
%% ...
}
\references{
\insertAllCited{}
}

\author{Michael Dewey}
\seealso{
See also \code{\link{plotp}}
}
\examples{
data(beckerp)
minimump(beckerp) # signif = FALSE, critp = 0.0102, minp = 0.016
data(teachexpect)
minimump(teachexpect) # crit 0.0207, note Becker says minp = 0.0011
wilkinsonp(c(0.223, 0.223), r = 2) # Birnbaum, just signif
data(validity)
minimump(validity) # minp = 0.00001, critp = 1.99 * 10^{-4}
minimump(c(0.0001, 0.0001, 0.9999, 0.9999)) # is significant
}
\keyword{ htest }

\name{meanp}
\alias{meanp}
\alias{print.meanp}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Combine p values by the mean p method}
\description{Combine \eqn{p} values by the mean p method}
\usage{
meanp(p)
\method{print}{meanp}(x, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{p}{\sigvec}
  \item{x}{An object of class \sQuote{\code{meanp}}}
  \item{...}{Other arguments to be passed through}
}
\details{
Defined as \deqn{z = (0.5 - \bar{p}) \sqrt{12k}}{z = (0.5 - mean(p)) * sqrt(12k)}
which is a standard normal and due to \insertCite{edgington72b;textual}{metap}
\insertNoCite{becker94}{metap}\insertNoCite{rosenthal78}{metap}

\lele{four}

\plotmethod
}
\value{
An object of class \sQuote{\code{meanp}} and \sQuote{\code{metap}},
a list with entries
  \item{z}{Value of \eqn{z}}
  \item{p}{Associated \eqn{p}-value}
  \item{validp}{The input vector with illegal values removed}
}
\references{
\insertAllCited{}
}
\author{Michael Dewey}

\seealso{
See also \code{\link{plotp}}
}
\examples{
data(rosenthal)
meanp(rosenthal$p) # 2.17, p = 0.015 one tailed
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ htest }

\name{metap-package}
\alias{metap-package}
\alias{metap}
\docType{package}
\title{\packageTitle{metap}}
\description{\packageDescription{metap}}
\details{
%\packageDESCRIPTION{metap}
\packageIndices{metap}

Provides a number of ways in which significance levels may be combined
in a meta-analysis and includes most ot the methods
in \insertCite{becker94;textual}{metap}.
It includes a number of datasets taken from the literature. 
It also provides a display and
an informal graphical test due to Schweder and Spjotvoll \insertCite{schweder82}{metap}
and the lowest slope line of Benjamini and
Hochberg \insertCite{benjamini00}{metap}.

There is a vignette.
Some further documentation and,
possibly, a development version
may be seen at
\url{http://www.dewey.myzen.co.uk/meta/meta.html}.

}
\author{\packageAuthor{metap}

Maintainer: \packageMaintainer{metap}
}
\references{
\insertAllCited{}
}
\keyword{ package }
\seealso{
The issue of meta-analysis of signficance levels is not completely
unconnected with the topic of adjustment for multiple
comparisons as in for example \code{\link{p.adjust}}
}

\name{votep}
\alias{votep}
\alias{print.votep}
\title{Combine p-values by the vote counting method}
\description{
Combine \mjseqn{p}-values by the vote counting method\loadmathjax
}
\usage{
votep(p, alpha = 0.5)
\method{print}{votep}(x, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{p}{\sigvec}
  \item{alpha}{A value defining the significance value limits, see Details}
  \item{x}{An object of class \sQuote{\code{votep}}}
  \item{...}{Other arguments to be passed through}
}
\details{By default splits the \mjseqn{p}-values at 0.5 with those below
counting as positive and those above counting as
negative.
However setting alpha
allows specifying a neutral zone.
In that case values between alpha and 1 - alpha will be
assumed to be neutral and only those outside the zone
counted as positive or negative.
If the value of alpha is greater than unity
it is assumed to be a percentage.

\ltlt{two}

\plotmethod
}
\value{An object of class \sQuote{\code{votep}}
and \sQuote{\code{votep}}, a list with entries
  \item{p}{\mjseqn{p} value}
  \item{pos}{Number of positives}
  \item{neg}{Number of negatives}
  \item{alpha}{The cut--off significance value lying in the range 0 to 1}
  \item{validp}{The input vector with illegal values removed}
}
\references{
\insertRef{becker94}{metap}
}

\author{Michael Dewey}

\seealso{
See also \code{\link{plotp}}
}
\examples{
data(dat.metap)
beckerp <- dat.metap$beckerp
votep(beckerp)
}
\keyword{ htest }

\name{logitp}
\alias{logitp}
\alias{print.logitp}
\title{Combine p values using the logit method}
\description{
Combine \mjseqn{p} values using logit method\loadmathjax
}
\usage{
logitp(p, log.p = FALSE)
\method{print}{logitp}(x, ...)
}
\arguments{
  \item{p}{\sigvec}
  \item{log.p}{\logp}
  \item{x}{An object of class \sQuote{\code{logitp}}}
  \item{...}{Other arguments to be passed through}
}
\details{
Defined as \mjdeqn{t = - \frac{\sum_{i=1}^k \log\frac{p_i}{1 - p_i}}{C}}{%
t = - sum log(p / (1 - p)) / C}
where
\mjdeqn{C = \sqrt\frac{k \pi^2 (5 k + 2)}{3(5 k + 4)}}{C = sqrt((k pi^2 (5 k + 2)) / (3(5 k + 4)))}
and \mjseqn{k} is the number of studies.

\ltlt{two}

\plotmethod
}
\value{
An object of class \sQuote{\code{logitp}} and
      \sQuote{\code{metap}}, a list with entries
  \item{t}{Value of Student's \mjseqn{t}}
  \item{df}{Associated degrees of freedom}
  \item{p}{Associated \mjseqn{p}-value}
  \item{validp}{The input vector with illegal values removed}
}
\references{
\insertRef{becker94}{metap}
}
\author{Michael Dewey}
\seealso{
See also \code{\link{plotp}}
}
\examples{
data(dat.metap)
teachexpect <- dat.metap$teachexpect
logitp(teachexpect) # t = 2.763, df = 99, p = 0.0034, from Becker
beckerp <- dat.metap$beckerp
logitp(beckerp) # t = 1.62, df = 29, NS, from Becker
validity <- dat.metap$validity$p
logitp(validity) # t = 9.521, df = 104, p = 3.89 * 10^{-16}
all.equal(exp(logitp(validity, log.p = TRUE)$p), logitp(validity)$p)
}
\keyword{ htest }

\name{schweder}
\alias{schweder}
\title{Schweder and Spjotvoll plot}
\description{Produces the plot suggested by Schweder and Spjotvoll to display
a collection of \eqn{p}-values and also optionally draws the
lowest slope line suggested by Benjamini and Hochberg
}
\usage{
schweder(p, xlab = "Rank of p", ylab = "p", drawline = NULL,
   bh.lwd = 1, bh.lty = "solid", bh.col = "black",
   ls.control = list(frac = NULL),
   ls.lwd = 1, ls.lty = "dotted", ls.col = "black",
   ab.control = list(a = NULL, b = NULL),
   ab.lwd = 1, ab.lty = "dashed", ab.col = "black", ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{p}{A vector of \eqn{p}-values}
  \item{xlab}{Label for plot \eqn{x}-axis}
  \item{ylab}{Label for plot \eqn{y}-axis}
  \item{drawline}{Which line, if any, to draw. See Details}
  \item{bh.lwd}{Width of the lowest slope line}
  \item{bh.lty}{Line type of the lowest slope line}
  \item{bh.col}{Colour of the lowest slope line}
  \item{ls.control}{A list containing a named element \code{frac}. See Details}
  \item{ls.lwd}{Width of the least squares line}
  \item{ls.lty}{Line type of the least squares line}
  \item{ls.col}{Colour of the least squares line}
  \item{ab.control}{A list containing two elements \code{a} and \code{b}. See Details}
  \item{ab.lwd}{Width of the specified line}
  \item{ab.lty}{Line type of the specified line}
  \item{ab.col}{Colour of the specified line}
  \item{\dots}{Further parameters to be passed through to plot}
}
\details{After removing invalid \eqn{p}-values plots them.
Any graphics parameters passed in \code{...} affect this part
of the plot only.
Note that the axes are chosen according to the scheme of Banjamini and
Hochberg, Schweder and Spjotvoll used a different system.

By setting \code{drawline} appropriately up to three
lines may be drawn.

If \code{drawline} includes the string \code{"bh"}
the lowest slope line of Benjaimin and Hochberg is drawn.
No further parameters are needed here but the characteristics of the
line may be set: width, line type, and colour.

If \code{drawline} includes the string \code{"ls"}
a least squares line is drawn passing through the point
\eqn{k+1, 1}.
The parameter \code{frac} specified what fraction of the
values be used for this and may need experimentation
to obtain a suitable line.
The characteristics of the
line may be set: width, line type, and colour.

If \code{drawline} includes the string \code{"ab"}
a user specified line is drawn.
The parameters \code{a} and \code{b} specify the intercept
and slope. 
The characteristics of the
line may be set: width, line type, and colour.
}
\value{
A list containing
  \item{p}{The values of \eqn{p} plotted}
  \item{bh.params}{A list containing at least items \code{a} and \code{b}
      the intercept and slope of the line drawn. This is only included if the
      lowest slope line was drawn}
  \item{ls.params}{A list containing at least items \code{a} and \code{b}
      the intercept and slope of the line drawn. This is only included if the
      least squares line was drawn}
  \item{ab.params}{A list containing at least items \code{a} and \code{b}
      the intercept and slope of the line drawn. This is only included if the
      user specified line was drawn}
}
\references{
\insertRef{becker94}{metap}

\insertRef{benjamini00}{metap}

\insertRef{schweder82}{metap}
}
\author{Michael Dewey}

\examples{
   data(teachexpect)
   schweder(teachexpect)
}
\keyword{ hplot }

\name{plotp}
\alias{plotp}
\title{Q-Q plot of p-values}
\description{
Produces a Q--Q plot of the p-values supplied\loadmathjax
}
\usage{
plotp(pvals, plotversion = "qqconf", ...)
}
\arguments{
  \item{pvals}{A vector of \mjseqn{p}--values}
  \item{plotversion}{Whether to use the qqconf version or the original}
  \item{\dots}{Other parameters to pass through to \code{qqplot}
  and \code{qqline}
}
}
\details{After eliminating out of range \mjseqn{p}--values
and missing values produces a Q--Q plot.
If the package \code{qqconf} is unavailable or
\code{plotversion} is set other than "qqconf"
then the plot uses the function from base graphics.
The line shown is
through the quantiles (by default the 0.25 and 0.75).

If the package \code{qqconf} is available then the plot
uses that package's function and as well as the Q--Q plot
produces a confidence region.
Parameters can be passed to \code{qq_conf_plot} to
exert further control over the plot.

If \code{plotversion} is set to "qqconf"
but package \code{qqconf} is not available
then a warning is given and no plot is produced.

}
\value{
Invisibly returns a list consisting of
  \item{validp}{The valid \mjseqn{p}--values}
}
\author{Michael Dewey}
\seealso{
\code{\link{qqplot}} and \code{\link{qqline}} for
the original plot and \code{\link[qqconf]{qq_conf_plot}} for
the new version.
}
\examples{
data(dat.metap)
cholest <- dat.metap$cholest$p
plotp(cholest)
}
\keyword{hplot}
